"""
Plotting functions for nIR_precision.

"""
import sys

import matplotlib.pyplot as plt
import numpy as np
from matplotlib import rc

# to remove labels in one tick
from matplotlib.ticker import MaxNLocator

from eniric.utilities import band_selector, rv_cumulative

# set stuff for latex usage
rc("text", usetex=True)


def plot_atmosphere_model(wav_atm, flux_atm, mask_atm):
    """Plot atmospheric transmission model for each band.

    This is run when plot_atom=True in calculate_prec()"""
    # identify non-masked pixels
    selected_transmission = wav_atm[mask_atm]
    dummy_vector = np.ones_like(selected_transmission)
    """
    plt.figure(1)
    plt.xlabel(r"wavelength [$\mu$m])")
    plt.ylabel(r"Transmission [ ] ")
    plt.plot(wav_atm, flux_atm, color='k')
    plt.vlines(selected_transmission, 0.8, 1.0, colors="b")
    plt.xlim(wav_atm[0], wav_atm[-1])
    plt.ylim(0.0, 1.0)
    # plt.legend(loc='best')
    plt.show()
    plt.close()
    """
    # do the same per band
    wav_atm_Z, flux_atm_Z = band_selector(wav_atm, flux_atm, "Z")
    wav_atm_Y, flux_atm_Y = band_selector(wav_atm, flux_atm, "Y")
    wav_atm_J, flux_atm_J = band_selector(wav_atm, flux_atm, "J")
    wav_atm_H, flux_atm_H = band_selector(wav_atm, flux_atm, "H")
    wav_atm_K, flux_atm_K = band_selector(wav_atm, flux_atm, "K")

    wav_mask_Z, __ = band_selector(selected_transmission, dummy_vector, "Z")
    wav_mask_Y, __ = band_selector(selected_transmission, dummy_vector, "Y")
    wav_mask_J, __ = band_selector(selected_transmission, dummy_vector, "J")
    wav_mask_H, __ = band_selector(selected_transmission, dummy_vector, "H")
    wav_mask_K, __ = band_selector(selected_transmission, dummy_vector, "K")

    print(
        "Z:",
        len(wav_mask_Z) / float(len(wav_atm_Z)),
        np.average(flux_atm_Z),
        np.median(flux_atm_Z),
    )
    print(
        "Y:",
        len(wav_mask_Y) / float(len(wav_atm_Y)),
        np.average(flux_atm_Y),
        np.median(flux_atm_Y),
    )
    print(
        "J:",
        len(wav_mask_J) / float(len(wav_atm_J)),
        np.average(flux_atm_J),
        np.median(flux_atm_J),
    )
    print(
        "H:",
        len(wav_mask_H) / float(len(wav_atm_H)),
        np.average(flux_atm_H),
        np.median(flux_atm_H),
    )
    print(
        "K:",
        len(wav_mask_K) / float(len(wav_atm_K)),
        np.average(flux_atm_K),
        np.median(flux_atm_K),
    )

    f, (ax1, ax2, ax3, ax4, ax5) = plt.subplots(5, sharex=False, sharey=False)

    ax1.plot(wav_atm_Z, flux_atm_Z, color="k")
    # ax1.vlines(selected_transmission, 0.0, 0.2, colors="b")
    ax1.set_xlim([wav_atm_Z[0], wav_atm_Z[-1]])
    ax1.yaxis.set_major_locator(
        MaxNLocator(nbins=len(ax1.get_yticklabels()), prune="lower")
    )
    ax1.text(
        0.9,
        0.8,
        " Z band ",
        horizontalalignment="center",
        verticalalignment="center",
        transform=ax1.transAxes,
        size=12,
        color="black",
        bbox=dict(facecolor="white"),
    )

    ax2.plot(wav_atm_Y, flux_atm_Y, color="k")
    # ax2.vlines(selected_transmission, 0.0, 0.2, colors="b")
    ax2.set_xlim([wav_atm_Y[0], wav_atm_Y[-1]])
    ax2.yaxis.set_major_locator(
        MaxNLocator(nbins=len(ax2.get_yticklabels()), prune="lower")
    )
    ax2.text(
        0.9,
        0.8,
        " Y band ",
        horizontalalignment="center",
        verticalalignment="center",
        transform=ax2.transAxes,
        size=12,
        color="black",
        bbox=dict(facecolor="white"),
    )

    ax3.plot(wav_atm_J, flux_atm_J, color="k")
    # ax3.vlines(selected_transmission, 0.0, 0.2, colors="b")
    ax3.set_xlim([wav_atm_J[0], wav_atm_J[-1]])
    ax3.yaxis.set_major_locator(
        MaxNLocator(nbins=len(ax3.get_yticklabels()), prune="upper")
    )
    ax3.text(
        0.9,
        0.8,
        " J band",
        horizontalalignment="center",
        verticalalignment="center",
        transform=ax3.transAxes,
        size=12,
        color="black",
        bbox=dict(facecolor="white"),
    )

    ax4.plot(wav_atm_H, flux_atm_H, color="k")
    # ax4.vlines(selected_transmission, 0.0, 0.2, colors="b")
    ax4.set_xlim([wav_atm_H[0], wav_atm_H[-1]])
    ax4.yaxis.set_major_locator(
        MaxNLocator(nbins=len(ax4.get_yticklabels()), prune="lower")
    )
    ax4.text(
        0.9,
        0.8,
        " H band ",
        horizontalalignment="center",
        verticalalignment="center",
        transform=ax4.transAxes,
        size=12,
        color="black",
        bbox=dict(facecolor="white"),
    )

    ax5.plot(wav_atm_K, flux_atm_K, color="k")
    # ax5.vlines(selected_transmission, 0.0, 0.2, colors="b")
    ax5.set_xlim([wav_atm_K[0], wav_atm_K[-1]])
    ax5.yaxis.set_major_locator(
        MaxNLocator(nbins=len(ax5.get_yticklabels()), prune="upper")
    )
    ax5.text(
        0.9,
        0.8,
        " K band ",
        horizontalalignment="center",
        verticalalignment="center",
        transform=ax5.transAxes,
        size=12,
        color="black",
        bbox=dict(facecolor="white"),
    )

    plt.xlabel(r"wavelength [$\mu$m]")
    f.text(
        0.06,
        0.5,
        r"Atmospheric Transmission [ ]",
        ha="center",
        va="center",
        rotation="vertical",
    )
    plt.show()
    # f.savefig('AtmosphericTransmission.pdf', format='pdf')

    plt.close()
    sys.exit(0)


def plot_stellar_spectum(
    wav_stellar, flux_stellar, wav_atm_selected, mask_atm_selected
):
    """Plot the stellar spectrum as considered."""
    selected_transmission_stellar = wav_atm_selected[mask_atm_selected]

    plt.figure(1)
    plt.xlabel(r"wavelength [$\mu$m])")
    plt.ylabel(r"Flux_stellar [ ] ")
    plt.plot(wav_stellar, flux_stellar, color="k")
    plt.vlines(
        selected_transmission_stellar,
        np.min(flux_stellar),
        0.3 * np.max(flux_stellar),
        colors="b",
    )
    plt.xlim(wav_stellar[0], wav_stellar[-1])
    plt.ylim(
        np.min(flux_stellar) - 0.1 * (np.max(flux_stellar) - np.min(flux_stellar)),
        np.max(flux_stellar) + 0.1 * (np.max(flux_stellar) - np.min(flux_stellar)),
    )
    # plt.legend(loc='best')
    plt.show()
    plt.close()


def plot_nIR_flux():
    """Plot_flux code from nIR_precision"""
    # print the plot for flux comparison
    f, (ax1, ax2, ax3, ax4) = plt.subplots(4, sharex=True, sharey=True)

    ax1.ticklabel_format(style="sci", axis="y", scilimits=(0, 0))

    ax1.plot(wav_plot_m0[0], np.array(flux_plot_m0[0]), color="0.1")
    ax1.plot(wav_plot_m0[1], np.array(flux_plot_m0[1]), color="0.1")
    ax1.plot(wav_plot_m0[2], np.array(flux_plot_m0[2]), color="0.1")
    ax1.plot(wav_plot_m0[3], np.array(flux_plot_m0[3]), color="0.1")
    ax1.plot(wav_plot_m0[4], np.array(flux_plot_m0[4]), color="0.1")
    ax1.text(
        0.9,
        0.8,
        "M0",
        horizontalalignment="center",
        verticalalignment="center",
        transform=ax1.transAxes,
        size=12,
    )
    ax1.plot(
        wav_m0,
        (flux_m0 / flux_m0[0]) * flux_plot_m0[0][1000],
        color="0.1",
        linestyle="--",
    )

    ax2.plot(wav_plot_m3[0], np.array(flux_plot_m3[0]), color="0.3")
    ax2.plot(wav_plot_m3[1], np.array(flux_plot_m3[1]), color="0.3")
    ax2.plot(wav_plot_m3[2], np.array(flux_plot_m3[2]), color="0.3")
    ax2.plot(wav_plot_m3[3], np.array(flux_plot_m3[3]), color="0.3")
    ax2.plot(wav_plot_m3[4], np.array(flux_plot_m3[4]), color="0.3")
    ax2.text(
        0.9,
        0.8,
        "M3",
        horizontalalignment="center",
        verticalalignment="center",
        transform=ax2.transAxes,
        size=12,
    )
    ax2.plot(
        wav_m3,
        (flux_m3 / flux_m3[0]) * flux_plot_m3[0][1000],
        color="0.1",
        linestyle="--",
    )

    ax2.get_yaxis().get_offset_text().set_visible(False)  # remove offset from the yaxis
    ax2.yaxis.set_major_locator(
        MaxNLocator(nbins=4, prune="upper")
    )  # remove upper element from yaxis
    # ax2.yaxis.set_major_locator(MaxNLocator(nbins=len(ax2.get_yticklabels()), prune='upper'))   # remove upper element from yaxis

    ax3.plot(wav_plot_m6[0], np.array(flux_plot_m6[0]), color="0.4")
    ax3.plot(wav_plot_m6[1], np.array(flux_plot_m6[1]), color="0.4")
    ax3.plot(wav_plot_m6[2], np.array(flux_plot_m6[2]), color="0.4")
    ax3.plot(wav_plot_m6[3], np.array(flux_plot_m6[3]), color="0.4")
    ax3.plot(wav_plot_m6[4], np.array(flux_plot_m6[4]), color="0.4")
    ax3.text(
        0.9,
        0.8,
        "M6",
        horizontalalignment="center",
        verticalalignment="center",
        transform=ax3.transAxes,
        size=12,
    )
    ax3.plot(
        wav_m6,
        (flux_m6 / flux_m6[0]) * flux_plot_m6[0][1000] * 1.2,
        color="0.1",
        linestyle="--",
    )

    ax3.get_yaxis().get_offset_text().set_visible(False)  # remove offset from the yaxis

    ax4.plot(wav_plot_m9[0], np.array(flux_plot_m9[0]), color="0.6")
    ax4.plot(wav_plot_m9[1], np.array(flux_plot_m9[1]), color="0.6")
    ax4.plot(wav_plot_m9[2], np.array(flux_plot_m9[2]), color="0.6")
    ax4.plot(wav_plot_m9[3], np.array(flux_plot_m9[3]), color="0.6")
    ax4.plot(wav_plot_m9[4], np.array(flux_plot_m9[4]), color="0.6")
    ax4.text(
        0.9,
        0.8,
        "M9",
        horizontalalignment="center",
        verticalalignment="center",
        transform=ax4.transAxes,
        size=12,
    )
    ax4.plot(
        wav_m9,
        (flux_m9 / flux_m9[0]) * flux_plot_m9[0][1000] * 1.4,
        color="0.1",
        linestyle="--",
    )

    ax4.get_yaxis().get_offset_text().set_visible(False)  # remove offset from the yaxis

    f.subplots_adjust(hspace=0)
    plt.setp([a.get_xticklabels() for a in f.axes[:-1]], visible=False)
    f.text(0.06, 0.5, r"Flux [ ]", ha="center", va="center", rotation="vertical")
    plt.xlabel(r"wavelength [$\mu$m]")

    plt.show()
    plt.close()

    # print the Z band spectra
    f, (ax1, ax2, ax3, ax4) = plt.subplots(4, sharex=True, sharey=True)
    plt.xlabel(r"wavelength [$\mu$m]")
    f.text(
        0.06, 0.5, r"Flux in Z band [ ]", ha="center", va="center", rotation="vertical"
    )

    ax1.ticklabel_format(style="sci", axis="y", scilimits=(0, 0))
    ax1.set_xlim(np.min(wav_plot_m0[0]), np.max(wav_plot_m0[0]))
    ax1.set_ylim(0.0, 5.0e3)

    ax1.plot(wav_plot_m0[0], np.array(flux_plot_m0[0]), color="0.1", label="M0")
    ax1.text(
        0.9,
        0.8,
        "M0",
        horizontalalignment="center",
        verticalalignment="center",
        transform=ax1.transAxes,
        size=12,
    )
    ax1.yaxis.set_major_locator(MaxNLocator(nbins=4, prune="both"))

    ax2.plot(wav_plot_m3[0], np.array(flux_plot_m3[0]), color="0.3", label="M3")
    ax2.text(
        0.9,
        0.8,
        "M3",
        horizontalalignment="center",
        verticalalignment="center",
        transform=ax2.transAxes,
        size=12,
    )
    ax2.get_yaxis().get_offset_text().set_visible(False)  # remove offset from the yaxis
    ax2.yaxis.set_major_locator(
        MaxNLocator(nbins=4, prune="both")
    )  # remove upper element from yaxis

    ax3.plot(wav_plot_m6[0], np.array(flux_plot_m6[0]), color="0.4", label="M6")
    ax3.text(
        0.9,
        0.8,
        "M6",
        horizontalalignment="center",
        verticalalignment="center",
        transform=ax3.transAxes,
        size=12,
    )
    ax3.get_yaxis().get_offset_text().set_visible(False)  # remove offset from the yaxis
    ax3.yaxis.set_major_locator(MaxNLocator(nbins=4, prune="both"))

    ax4.plot(wav_plot_m9[0], np.array(flux_plot_m9[0]), color="0.6", label="M9")
    ax4.text(
        0.9,
        0.8,
        "M9",
        horizontalalignment="center",
        verticalalignment="center",
        transform=ax4.transAxes,
        size=12,
    )
    ax4.get_yaxis().get_offset_text().set_visible(False)  # remove offset from the yaxis
    ax4.yaxis.set_major_locator(MaxNLocator(nbins=4, prune="both"))

    f.subplots_adjust(hspace=0)
    plt.setp([a.get_xticklabels() for a in f.axes[:-1]], visible=False)
    plt.show()
    plt.close()

    # print the Y band spectra
    f, (ax1, ax2, ax3, ax4) = plt.subplots(4, sharex=True, sharey=True)
    plt.xlabel(r"wavelength [$\mu$m]")
    f.text(
        0.06, 0.5, r"Flux in Y band [ ]", ha="center", va="center", rotation="vertical"
    )

    ax1.ticklabel_format(style="sci", axis="y", scilimits=(0, 0))

    ax1.plot(wav_plot_m0[1], np.array(flux_plot_m0[1]), color="0.1", label="M0")
    ax1.text(
        0.9,
        0.8,
        "M0",
        horizontalalignment="center",
        verticalalignment="center",
        transform=ax1.transAxes,
        size=12,
    )

    ax2.plot(wav_plot_m3[1], np.array(flux_plot_m3[1]), color="0.3", label="M3")
    ax2.text(
        0.9,
        0.8,
        "M3",
        horizontalalignment="center",
        verticalalignment="center",
        transform=ax2.transAxes,
        size=12,
    )
    ax2.get_yaxis().get_offset_text().set_visible(False)  # remove offset from the yaxis
    ax2.yaxis.set_major_locator(
        MaxNLocator(nbins=4, prune="upper")
    )  # remove upper element from yaxis

    ax3.plot(wav_plot_m6[1], np.array(flux_plot_m6[1]), color="0.4", label="M6")
    ax3.text(
        0.9,
        0.8,
        "M6",
        horizontalalignment="center",
        verticalalignment="center",
        transform=ax3.transAxes,
        size=12,
    )
    ax3.get_yaxis().get_offset_text().set_visible(False)  # remove offset from the yaxis

    ax4.plot(wav_plot_m9[1], np.array(flux_plot_m9[1]), color="0.6", label="M9")
    ax4.text(
        0.9,
        0.8,
        "M9",
        horizontalalignment="center",
        verticalalignment="center",
        transform=ax4.transAxes,
        size=12,
    )
    ax4.get_yaxis().get_offset_text().set_visible(False)  # remove offset from the yaxis

    f.subplots_adjust(hspace=0)
    plt.setp([a.get_xticklabels() for a in f.axes[:-1]], visible=False)
    plt.show()
    plt.close()

    # print the J band spectra
    f, (ax1, ax2, ax3, ax4) = plt.subplots(4, sharex=True, sharey=True)
    plt.xlabel(r"wavelength [$\mu$m]")
    f.text(
        0.06, 0.5, r"Flux in J band [ ]", ha="center", va="center", rotation="vertical"
    )

    ax1.ticklabel_format(style="sci", axis="y", scilimits=(0, 0))
    ax1.set_xlim(np.min(wav_plot_m0[2]), np.max(wav_plot_m0[2]))

    ax1.plot(wav_plot_m0[2], np.array(flux_plot_m0[2]), color="0.1", label="M0")
    ax1.text(
        0.9,
        0.8,
        "M0",
        horizontalalignment="center",
        verticalalignment="center",
        transform=ax1.transAxes,
        size=12,
    )

    ax2.plot(wav_plot_m3[2], np.array(flux_plot_m3[2]), color="0.3", label="M3")
    ax2.text(
        0.9,
        0.8,
        "M3",
        horizontalalignment="center",
        verticalalignment="center",
        transform=ax2.transAxes,
        size=12,
    )
    ax2.get_yaxis().get_offset_text().set_visible(False)  # remove offset from the yaxis
    ax2.yaxis.set_major_locator(
        MaxNLocator(nbins=4, prune="upper")
    )  # remove upper element from yaxis

    ax3.plot(wav_plot_m6[2], np.array(flux_plot_m6[2]), color="0.4", label="M6")
    ax3.text(
        0.9,
        0.8,
        "M6",
        horizontalalignment="center",
        verticalalignment="center",
        transform=ax3.transAxes,
        size=12,
    )
    ax3.get_yaxis().get_offset_text().set_visible(False)  # remove offset from the yaxis

    ax4.plot(wav_plot_m9[2], np.array(flux_plot_m9[2]), color="0.6", label="M9")
    ax4.text(
        0.9,
        0.8,
        "M9",
        horizontalalignment="center",
        verticalalignment="center",
        transform=ax4.transAxes,
        size=12,
    )
    ax4.get_yaxis().get_offset_text().set_visible(False)  # remove offset from the yaxis

    f.subplots_adjust(hspace=0)
    plt.setp([a.get_xticklabels() for a in f.axes[:-1]], visible=False)
    plt.show()
    plt.close()

    # print the H band spectra
    f, (ax1, ax2, ax3, ax4) = plt.subplots(4, sharex=True, sharey=True)
    plt.xlabel(r"wavelength [$\mu$m]")
    f.text(
        0.06, 0.5, r"Flux in H band [ ]", ha="center", va="center", rotation="vertical"
    )

    ax1.ticklabel_format(style="sci", axis="y", scilimits=(0, 0))

    ax1.plot(wav_plot_m0[3], np.array(flux_plot_m0[3]), color="0.1", label="M0")
    ax1.text(
        0.9,
        0.8,
        "M0",
        horizontalalignment="center",
        verticalalignment="center",
        transform=ax1.transAxes,
        size=12,
    )

    ax2.plot(wav_plot_m3[3], np.array(flux_plot_m3[3]), color="0.3", label="M3")
    ax2.text(
        0.9,
        0.8,
        "M3",
        horizontalalignment="center",
        verticalalignment="center",
        transform=ax2.transAxes,
        size=12,
    )
    ax2.get_yaxis().get_offset_text().set_visible(False)  # remove offset from the yaxis
    ax2.yaxis.set_major_locator(
        MaxNLocator(nbins=4, prune="upper")
    )  # remove upper element from yaxis

    ax3.plot(wav_plot_m6[3], np.array(flux_plot_m6[3]), color="0.4", label="M6")
    ax3.text(
        0.9,
        0.8,
        "M6",
        horizontalalignment="center",
        verticalalignment="center",
        transform=ax3.transAxes,
        size=12,
    )
    ax3.get_yaxis().get_offset_text().set_visible(False)  # remove offset from the yaxis

    ax4.plot(wav_plot_m9[3], np.array(flux_plot_m9[3]), color="0.6", label="M9")
    ax4.text(
        0.9,
        0.8,
        "M9",
        horizontalalignment="center",
        verticalalignment="center",
        transform=ax4.transAxes,
        size=12,
    )
    ax4.get_yaxis().get_offset_text().set_visible(False)  # remove offset from the yaxis

    f.subplots_adjust(hspace=0)
    plt.setp([a.get_xticklabels() for a in f.axes[:-1]], visible=False)
    plt.show()
    plt.close()

    # print the K band spectra
    f, (ax1, ax2, ax3, ax4) = plt.subplots(4, sharex=True, sharey=True)
    plt.xlabel(r"wavelength [$\mu$m]")
    f.text(
        0.06, 0.5, r"Flux in K band [ ]", ha="center", va="center", rotation="vertical"
    )

    ax1.set_xlim([np.min(wav_plot_m0[4]), np.max(wav_plot_m0[4])])
    ax1.ticklabel_format(style="sci", axis="y", scilimits=(0, 0))

    ax1.plot(wav_plot_m0[4], np.array(flux_plot_m0[4]), color="0.1", label="M0")
    ax1.text(
        0.9,
        0.8,
        "M0",
        horizontalalignment="center",
        verticalalignment="center",
        transform=ax1.transAxes,
        size=12,
    )

    ax2.plot(wav_plot_m3[4], np.array(flux_plot_m3[4]), color="0.3", label="M3")
    ax2.text(
        0.9,
        0.8,
        "M3",
        horizontalalignment="center",
        verticalalignment="center",
        transform=ax2.transAxes,
        size=12,
    )
    ax2.get_yaxis().get_offset_text().set_visible(False)  # remove offset from the yaxis
    ax2.yaxis.set_major_locator(
        MaxNLocator(nbins=4, prune="upper")
    )  # remove upper element from yaxis

    ax3.plot(wav_plot_m6[4], np.array(flux_plot_m6[4]), color="0.4", label="M6")
    ax3.text(
        0.9,
        0.8,
        "M6",
        horizontalalignment="center",
        verticalalignment="center",
        transform=ax3.transAxes,
        size=12,
    )
    ax3.get_yaxis().get_offset_text().set_visible(False)  # remove offset from the yaxis

    ax4.plot(wav_plot_m9[4], np.array(flux_plot_m9[4]), color="0.6", label="M9")
    ax4.text(
        0.9,
        0.8,
        "M9",
        horizontalalignment="center",
        verticalalignment="center",
        transform=ax4.transAxes,
        size=12,
    )
    ax4.get_yaxis().get_offset_text().set_visible(False)  # remove offset from the yaxis

    f.subplots_adjust(hspace=0)
    plt.setp([a.get_xticklabels() for a in f.axes[:-1]], visible=False)

    plt.show()
    plt.close()


def plot_paper_plots():
    """Print the paper plots.

    In every plot we will assume sample=3 and vsini=1
    y = RVprec between prec3 and prec2
    x = different bands

    different panels for different spectral types

    different colors will represent the different resolutions
    """

    print("Results for vsini of 1.0 km/s")
    # preparation of data: plot1
    y1_60k = [
        results["M0-Z-1.0-60k"],
        results["M0-Y-1.0-60k"],
        results["M0-J-1.0-60k"],
        results["M0-H-1.0-60k"],
        results["M0-K-1.0-60k"],
    ]
    y1_60k_top = [y[1] for y in y1_60k]
    y1_60k_bottom = [y[2] for y in y1_60k]

    y1_60k_lim = [y[0] for y in y1_60k]

    y1_80k = [
        results["M0-Z-1.0-80k"],
        results["M0-Y-1.0-80k"],
        results["M0-J-1.0-80k"],
        results["M0-H-1.0-80k"],
        results["M0-K-1.0-80k"],
    ]
    y1_80k_top = [y[1] for y in y1_80k]
    y1_80k_bottom = [y[2] for y in y1_80k]

    y1_80k_lim = [y[0] for y in y1_80k]

    y1_100k = [
        results["M0-Z-1.0-100k"],
        results["M0-Y-1.0-100k"],
        results["M0-J-1.0-100k"],
        results["M0-H-1.0-100k"],
        results["M0-K-1.0-100k"],
    ]
    y1_100k_top = [y[1] for y in y1_100k]
    y1_100k_bottom = [y[2] for y in y1_100k]

    y1_100k_lim = [y[0] for y in y1_100k]

    # preparation of data: plot2
    y2_60k = [
        results["M3-Z-1.0-60k"],
        results["M3-Y-1.0-60k"],
        results["M3-J-1.0-60k"],
        results["M3-H-1.0-60k"],
        results["M3-K-1.0-60k"],
    ]
    y2_60k_top = [y[1] for y in y2_60k]
    y2_60k_bottom = [y[2] for y in y2_60k]

    y2_60k_lim = [y[0] for y in y2_60k]

    y2_80k = [
        results["M3-Z-1.0-80k"],
        results["M3-Y-1.0-80k"],
        results["M3-J-1.0-80k"],
        results["M3-H-1.0-80k"],
        results["M3-K-1.0-80k"],
    ]
    y2_80k_top = [y[1] for y in y2_80k]
    y2_80k_bottom = [y[2] for y in y2_80k]

    y2_80k_lim = [y[0] for y in y2_80k]

    y2_100k = [
        results["M3-Z-1.0-100k"],
        results["M3-Y-1.0-100k"],
        results["M3-J-1.0-100k"],
        results["M3-H-1.0-100k"],
        results["M3-K-1.0-100k"],
    ]
    y2_100k_top = [y[1] for y in y2_100k]
    y2_100k_bottom = [y[2] for y in y2_100k]

    y2_100k_lim = [y[0] for y in y2_100k]

    # preparation of data: plot3
    y3_60k = [
        results["M6-Z-1.0-60k"],
        results["M6-Y-1.0-60k"],
        results["M6-J-1.0-60k"],
        results["M6-H-1.0-60k"],
        results["M6-K-1.0-60k"],
    ]
    y3_60k_top = [y[1] for y in y3_60k]
    y3_60k_bottom = [y[2] for y in y3_60k]

    y3_60k_lim = [y[0] for y in y3_60k]

    y3_80k = [
        results["M6-Z-1.0-80k"],
        results["M6-Y-1.0-80k"],
        results["M6-J-1.0-80k"],
        results["M6-H-1.0-80k"],
        results["M6-K-1.0-80k"],
    ]
    y3_80k_top = [y[1] for y in y3_80k]
    y3_80k_bottom = [y[2] for y in y3_80k]

    y3_80k_lim = [y[0] for y in y3_80k]

    y3_100k = [
        results["M6-Z-1.0-100k"],
        results["M6-Y-1.0-100k"],
        results["M6-J-1.0-100k"],
        results["M6-H-1.0-100k"],
        results["M6-K-1.0-100k"],
    ]
    y3_100k_top = [y[1] for y in y3_100k]
    y3_100k_bottom = [y[2] for y in y3_100k]

    y3_100k_lim = [y[0] for y in y3_100k]

    # preparation of data: plot4
    y4_60k = [
        results["M9-Z-1.0-60k"],
        results["M9-Y-1.0-60k"],
        results["M9-J-1.0-60k"],
        results["M9-H-1.0-60k"],
        results["M9-K-1.0-60k"],
    ]
    y4_60k_top = [y[1] for y in y4_60k]
    y4_60k_bottom = [y[2] for y in y4_60k]

    y4_60k_lim = [y[0] for y in y4_60k]

    y4_80k = [
        results["M9-Z-1.0-80k"],
        results["M9-Y-1.0-80k"],
        results["M9-J-1.0-80k"],
        results["M9-H-1.0-80k"],
        results["M9-K-1.0-80k"],
    ]
    y4_80k_top = [y[1] for y in y4_80k]
    y4_80k_bottom = [y[2] for y in y4_80k]

    y4_80k_lim = [y[0] for y in y4_80k]

    y4_100k = [
        results["M9-Z-1.0-100k"],
        results["M9-Y-1.0-100k"],
        results["M9-J-1.0-100k"],
        results["M9-H-1.0-100k"],
        results["M9-K-1.0-100k"],
    ]
    y4_100k_top = [y[1] for y in y4_100k]
    y4_100k_bottom = [y[2] for y in y4_100k]

    y4_100k_lim = [y[0] for y in y4_100k]

    datasets = [
        y1_60k_top,
        y1_60k_bottom,
        y1_80k_top,
        y1_80k_bottom,
        y1_100k_top,
        y1_100k_bottom,
        y2_60k_top,
        y2_60k_bottom,
        y2_80k_top,
        y2_80k_bottom,
        y2_100k_top,
        y2_100k_bottom,
        y3_60k_top,
        y3_60k_bottom,
        y3_80k_top,
        y3_80k_bottom,
        y3_100k_top,
        y3_100k_bottom,
        y4_60k_top,
        y4_60k_bottom,
        y4_80k_top,
        y4_80k_bottom,
        y4_100k_top,
        y4_100k_bottom,
    ]

    positiony_min, poisition_ymax = min_max_y(datasets)

    # data for correction plot
    y1_60k_vsini1 = (
        (np.array(y1_60k_top) - np.array(y1_60k_bottom))
        / np.array(y1_60k_bottom)
        * 100.0
    )
    y1_80k_vsini1 = (
        (np.array(y1_80k_top) - np.array(y1_80k_bottom))
        / np.array(y1_80k_bottom)
        * 100.0
    )
    y1_100k_vsini1 = (
        (np.array(y1_100k_top) - np.array(y1_100k_bottom))
        / np.array(y1_100k_bottom)
        * 100.0
    )

    y2_60k_vsini1 = (
        (np.array(y2_60k_top) - np.array(y2_60k_bottom))
        / np.array(y2_60k_bottom)
        * 100.0
    )
    y2_80k_vsini1 = (
        (np.array(y2_80k_top) - np.array(y2_80k_bottom))
        / np.array(y2_80k_bottom)
        * 100.0
    )
    y2_100k_vsini1 = (
        (np.array(y2_100k_top) - np.array(y2_100k_bottom))
        / np.array(y2_100k_bottom)
        * 100.0
    )

    y3_60k_vsini1 = (
        (np.array(y3_60k_top) - np.array(y3_60k_bottom))
        / np.array(y3_60k_bottom)
        * 100.0
    )
    y3_80k_vsini1 = (
        (np.array(y3_80k_top) - np.array(y3_80k_bottom))
        / np.array(y3_80k_bottom)
        * 100.0
    )
    y3_100k_vsini1 = (
        (np.array(y3_100k_top) - np.array(y3_100k_bottom))
        / np.array(y3_100k_bottom)
        * 100.0
    )

    y4_60k_vsini1 = (
        (np.array(y4_60k_top) - np.array(y4_60k_bottom))
        / np.array(y4_60k_bottom)
        * 100.0
    )
    y4_80k_vsini1 = (
        (np.array(y4_80k_top) - np.array(y4_80k_bottom))
        / np.array(y4_80k_bottom)
        * 100.0
    )
    y4_100k_vsini1 = (
        (np.array(y4_100k_top) - np.array(y4_100k_bottom))
        / np.array(y4_100k_bottom)
        * 100.0
    )

    fig = plt.figure(1)
    ax1 = fig.add_subplot(221)

    ax1.fill_between(
        range(1, len(bands) + 1), y1_60k_bottom, y1_60k_top, color="b", alpha=0.2
    )
    ax1.fill_between(
        range(1, len(bands) + 1), y1_80k_bottom, y1_80k_top, color="g", alpha=0.2
    )
    ax1.fill_between(
        range(1, len(bands) + 1), y1_100k_bottom, y1_100k_top, color="r", alpha=0.2
    )

    ax1.plot(range(1, len(bands) + 1), y1_60k_lim, color="b", linestyle="--")
    ax1.plot(range(1, len(bands) + 1), y1_80k_lim, color="g", linestyle="--")
    ax1.plot(range(1, len(bands) + 1), y1_100k_lim, color="r", linestyle="--")

    ax1.scatter(
        range(1, len(bands) + 1), y1_60k_bottom, marker="^", color="b", alpha=0.4
    )
    ax1.scatter(range(1, len(bands) + 1), y1_60k_top, marker="o", color="b", alpha=0.4)

    ax1.scatter(
        range(1, len(bands) + 1), y1_80k_bottom, marker="^", color="g", alpha=0.4
    )
    ax1.scatter(range(1, len(bands) + 1), y1_80k_top, marker="o", color="g", alpha=0.4)

    ax1.scatter(
        range(1, len(bands) + 1), y1_100k_bottom, marker="^", color="r", alpha=0.4
    )
    ax1.scatter(range(1, len(bands) + 1), y1_100k_top, marker="o", color="r", alpha=0.4)

    ax1.text(1.0, positiony_max, "M0", size=14)

    # ticks and labels
    # ax1.set_ylabel('Precision [m/s]')
    ax1.set_xlim(0.5, len(bands) + 0.5)
    ax1.set_xticks(range(1, len(bands) + 1))
    ax1.set_xticklabels([])
    ax1.set_ylim(
        positiony_min - 0.1 * (positiony_max - positiony_min),
        positiony_max + 0.1 * (positiony_max - positiony_min),
    )

    ax1.tick_params(axis="both", which="major", labelsize=15)

    ax2 = fig.add_subplot(222)

    ax2.fill_between(
        range(1, len(bands) + 1), y2_60k_bottom, y2_60k_top, color="b", alpha=0.2
    )
    ax2.fill_between(
        range(1, len(bands) + 1), y2_80k_bottom, y2_80k_top, color="g", alpha=0.2
    )
    ax2.fill_between(
        range(1, len(bands) + 1), y2_100k_bottom, y2_100k_top, color="r", alpha=0.2
    )

    ax2.plot(range(1, len(bands) + 1), y2_60k_lim, color="b", linestyle="--")
    ax2.plot(range(1, len(bands) + 1), y2_80k_lim, color="g", linestyle="--")
    ax2.plot(range(1, len(bands) + 1), y2_100k_lim, color="r", linestyle="--")

    ax2.scatter(
        range(1, len(bands) + 1), y2_60k_bottom, marker="^", color="b", alpha=0.4
    )
    ax2.scatter(range(1, len(bands) + 1), y2_60k_top, marker="o", color="b", alpha=0.4)

    ax2.scatter(
        range(1, len(bands) + 1), y2_80k_bottom, marker="^", color="g", alpha=0.4
    )
    ax2.scatter(range(1, len(bands) + 1), y2_80k_top, marker="o", color="g", alpha=0.4)

    ax2.scatter(
        range(1, len(bands) + 1), y2_100k_bottom, marker="^", color="r", alpha=0.4
    )
    ax2.scatter(range(1, len(bands) + 1), y2_100k_top, marker="o", color="r", alpha=0.4)

    ax2.text(1.0, positiony_max, "M3", size=14)

    # ticks and labels
    ax2.set_xlim(0.5, len(bands) + 0.5)
    ax2.set_xticks(range(1, len(bands) + 1))
    ax2.set_xticklabels([])
    ax2.set_yticklabels([])
    ax2.set_ylim(
        positiony_min - 0.1 * (positiony_max - positiony_min),
        positiony_max + 0.1 * (positiony_max - positiony_min),
    )

    ax2.tick_params(axis="both", which="major", labelsize=15)

    ax3 = fig.add_subplot(223)

    ax3.fill_between(
        range(1, len(bands) + 1), y3_60k_bottom, y3_60k_top, color="b", alpha=0.2
    )
    ax3.fill_between(
        range(1, len(bands) + 1), y3_80k_bottom, y3_80k_top, color="g", alpha=0.2
    )
    ax3.fill_between(
        range(1, len(bands) + 1), y3_100k_bottom, y3_100k_top, color="r", alpha=0.2
    )

    ax3.plot(range(1, len(bands) + 1), y3_60k_lim, color="b", linestyle="--")
    ax3.plot(range(1, len(bands) + 1), y3_80k_lim, color="g", linestyle="--")
    ax3.plot(range(1, len(bands) + 1), y3_100k_lim, color="r", linestyle="--")

    ax3.scatter(
        range(1, len(bands) + 1), y3_60k_bottom, marker="^", color="b", alpha=0.4
    )
    ax3.scatter(range(1, len(bands) + 1), y3_60k_top, marker="o", color="b", alpha=0.4)

    ax3.scatter(
        range(1, len(bands) + 1), y3_80k_bottom, marker="^", color="g", alpha=0.4
    )
    ax3.scatter(range(1, len(bands) + 1), y3_80k_top, marker="o", color="g", alpha=0.4)

    ax3.scatter(
        range(1, len(bands) + 1), y3_100k_bottom, marker="^", color="r", alpha=0.4
    )
    ax3.scatter(range(1, len(bands) + 1), y3_100k_top, marker="o", color="r", alpha=0.4)

    ax3.text(1.0, positiony_max, "M6", size=14)

    # ticks and labels
    # ax3.set_ylabel('Precision [m/s]')
    ax3.set_xlabel("Bands")
    ax3.set_xlim(0.5, len(bands) + 0.5)
    ax3.set_xticks(range(1, len(bands) + 1))
    ax3.set_xticklabels(bands)
    ax3.set_ylim(
        positiony_min - 0.1 * (positiony_max - positiony_min),
        positiony_max + 0.1 * (positiony_max - positiony_min),
    )

    ax3.tick_params(axis="both", which="major", labelsize=15)

    ax4 = fig.add_subplot(224)

    ax4.fill_between(
        range(1, len(bands) + 1), y4_60k_bottom, y4_60k_top, color="b", alpha=0.2
    )
    ax4.fill_between(
        range(1, len(bands) + 1), y4_80k_bottom, y4_80k_top, color="g", alpha=0.2
    )
    ax4.fill_between(
        range(1, len(bands) + 1), y4_100k_bottom, y4_100k_top, color="r", alpha=0.2
    )

    ax4.plot(range(1, len(bands) + 1), y4_60k_lim, color="b", linestyle="--")
    ax4.plot(range(1, len(bands) + 1), y4_80k_lim, color="g", linestyle="--")
    ax4.plot(range(1, len(bands) + 1), y4_100k_lim, color="r", linestyle="--")

    ax4.scatter(
        range(1, len(bands) + 1), y4_60k_bottom, marker="^", color="b", alpha=0.4
    )
    ax4.scatter(range(1, len(bands) + 1), y4_60k_top, marker="o", color="b", alpha=0.4)

    ax4.scatter(
        range(1, len(bands) + 1), y4_80k_bottom, marker="^", color="g", alpha=0.4
    )
    ax4.scatter(range(1, len(bands) + 1), y4_80k_top, marker="o", color="g", alpha=0.4)

    ax4.scatter(
        range(1, len(bands) + 1), y4_100k_bottom, marker="^", color="r", alpha=0.4
    )
    ax4.scatter(range(1, len(bands) + 1), y4_100k_top, marker="o", color="r", alpha=0.4)

    ax4.text(1.0, positiony_max, "M9", size=14)

    # ticks and labels
    ax4.set_xlabel("Bands")
    ax4.set_xlim(0.5, len(bands) + 0.5)
    ax4.set_xticks(range(1, len(bands) + 1))
    ax4.set_xticklabels(bands)
    ax4.set_yticklabels([])
    ax4.set_ylim(
        positiony_min - 0.1 * (positiony_max - positiony_min),
        positiony_max + 0.1 * (positiony_max - positiony_min),
    )

    ax4.tick_params(axis="both", which="major", labelsize=15)

    fig.text(
        0.06,
        0.5,
        r"Precision [m/s]",
        ha="center",
        va="center",
        rotation="vertical",
        size=14,
    )
    fig.subplots_adjust(hspace=0, wspace=0)

    plt.show()
    plt.close()

    """
    same plot for total precision
    """
    datasets = [
        y1_60k_top,
        y1_60k_bottom,
        y1_80k_top,
        y1_80k_bottom,
        y1_100k_top,
        y1_100k_bottom,
        y2_60k_top,
        y2_60k_bottom,
        y2_80k_top,
        y2_80k_bottom,
        y2_100k_top,
        y2_100k_bottom,
        y3_60k_top,
        y3_60k_bottom,
        y3_80k_top,
        y3_80k_bottom,
        y3_100k_top,
        y3_100k_bottom,
        y4_60k_top,
        y4_60k_bottom,
        y4_80k_top,
        y4_80k_bottom,
        y4_100k_top,
        y4_100k_bottom,
    ]

    positiony_min, poisition_ymax = min_max_y(datasets)

    bands_total = ["ZY", "ZYJ", "ZYJH", "ZYJHK"]
    fig = plt.figure(1)
    ax1 = fig.add_subplot(221)

    ax1.fill_between(
        range(1, len(bands_total) + 1),
        rv_cumulative(y1_60k_bottom),
        rv_cumulative(y1_60k_top),
        color="b",
        alpha=0.2,
    )
    ax1.fill_between(
        range(1, len(bands_total) + 1),
        rv_cumulative(y1_80k_bottom),
        rv_cumulative(y1_80k_top),
        color="g",
        alpha=0.2,
    )
    ax1.fill_between(
        range(1, len(bands_total) + 1),
        rv_cumulative(y1_100k_bottom),
        rv_cumulative(y1_100k_top),
        color="r",
        alpha=0.2,
    )

    ax1.scatter(
        range(1, len(bands_total) + 1),
        rv_cumulative(y1_60k_bottom),
        marker="^",
        color="b",
        alpha=0.4,
    )
    ax1.scatter(
        range(1, len(bands_total) + 1),
        rv_cumulative(y1_60k_top),
        marker="o",
        color="b",
        alpha=0.4,
    )

    ax1.scatter(
        range(1, len(bands_total) + 1),
        rv_cumulative(y1_80k_bottom),
        marker="^",
        color="g",
        alpha=0.4,
    )
    ax1.scatter(
        range(1, len(bands_total) + 1),
        rv_cumulative(y1_80k_top),
        marker="o",
        color="g",
        alpha=0.4,
    )

    ax1.scatter(
        range(1, len(bands_total) + 1),
        rv_cumulative(y1_100k_bottom),
        marker="^",
        color="r",
        alpha=0.4,
    )
    ax1.scatter(
        range(1, len(bands_total) + 1),
        rv_cumulative(y1_100k_top),
        marker="o",
        color="r",
        alpha=0.4,
    )

    ax1.text(1.0, positiony_max, "M0", size=14)

    # ticks and labels
    # ax1.set_ylabel('Precision [m/s]')
    ax1.set_xlim(0.5, len(bands_total) + 0.5)
    ax1.set_xticks(range(1, len(bands_total) + 1))
    ax1.set_xticklabels([])
    ax1.set_ylim(
        positiony_min - 0.1 * (positiony_max - positiony_min),
        positiony_max + 0.1 * (positiony_max - positiony_min),
    )

    ax1.tick_params(axis="both", which="major", labelsize=15)

    ax2 = fig.add_subplot(222)

    ax2.fill_between(
        range(1, len(bands_total) + 1),
        rv_cumulative(y2_60k_bottom),
        rv_cumulative(y2_60k_top),
        color="b",
        alpha=0.2,
    )
    ax2.fill_between(
        range(1, len(bands_total) + 1),
        rv_cumulative(y2_80k_bottom),
        rv_cumulative(y2_80k_top),
        color="g",
        alpha=0.2,
    )
    ax2.fill_between(
        range(1, len(bands_total) + 1),
        rv_cumulative(y2_100k_bottom),
        rv_cumulative(y2_100k_top),
        color="r",
        alpha=0.2,
    )

    ax2.scatter(
        range(1, len(bands_total) + 1),
        rv_cumulative(y2_60k_bottom),
        marker="^",
        color="b",
        alpha=0.4,
    )
    ax2.scatter(
        range(1, len(bands_total) + 1),
        rv_cumulative(y2_60k_top),
        marker="o",
        color="b",
        alpha=0.4,
    )

    ax2.scatter(
        range(1, len(bands_total) + 1),
        rv_cumulative(y2_80k_bottom),
        marker="^",
        color="g",
        alpha=0.4,
    )
    ax2.scatter(
        range(1, len(bands_total) + 1),
        rv_cumulative(y2_80k_top),
        marker="o",
        color="g",
        alpha=0.4,
    )

    ax2.scatter(
        range(1, len(bands_total) + 1),
        rv_cumulative(y2_100k_bottom),
        marker="^",
        color="r",
        alpha=0.4,
    )
    ax2.scatter(
        range(1, len(bands_total) + 1),
        rv_cumulative(y2_100k_top),
        marker="o",
        color="r",
        alpha=0.4,
    )

    ax2.text(1.0, positiony_max, "M3", size=14)

    # ticks and labels
    ax2.set_xlim(0.5, len(bands_total) + 0.5)
    ax2.set_xticks(range(1, len(bands_total) + 1))
    ax2.set_xticklabels([])
    ax2.set_yticklabels([])
    ax2.set_ylim(
        positiony_min - 0.1 * (positiony_max - positiony_min),
        positiony_max + 0.1 * (positiony_max - positiony_min),
    )

    ax2.tick_params(axis="both", which="major", labelsize=15)

    ax3 = fig.add_subplot(223)

    ax3.fill_between(
        range(1, len(bands_total) + 1),
        rv_cumulative(y3_60k_bottom),
        rv_cumulative(y3_60k_top),
        color="b",
        alpha=0.2,
    )
    ax3.fill_between(
        range(1, len(bands_total) + 1),
        rv_cumulative(y3_80k_bottom),
        rv_cumulative(y3_80k_top),
        color="g",
        alpha=0.2,
    )
    ax3.fill_between(
        range(1, len(bands_total) + 1),
        rv_cumulative(y3_100k_bottom),
        rv_cumulative(y3_100k_top),
        color="r",
        alpha=0.2,
    )

    ax3.scatter(
        range(1, len(bands_total) + 1),
        rv_cumulative(y3_60k_bottom),
        marker="^",
        color="b",
        alpha=0.4,
    )
    ax3.scatter(
        range(1, len(bands_total) + 1),
        rv_cumulative(y3_60k_top),
        marker="o",
        color="b",
        alpha=0.4,
    )

    ax3.scatter(
        range(1, len(bands_total) + 1),
        rv_cumulative(y3_80k_bottom),
        marker="^",
        color="g",
        alpha=0.4,
    )
    ax3.scatter(
        range(1, len(bands_total) + 1),
        rv_cumulative(y3_80k_top),
        marker="o",
        color="g",
        alpha=0.4,
    )

    ax3.scatter(
        range(1, len(bands_total) + 1),
        rv_cumulative(y3_100k_bottom),
        marker="^",
        color="r",
        alpha=0.4,
    )
    ax3.scatter(
        range(1, len(bands_total) + 1),
        rv_cumulative(y3_100k_top),
        marker="o",
        color="r",
        alpha=0.4,
    )

    ax3.text(1.0, positiony_max, "M6", size=14)

    # ticks and labels
    # ax3.set_ylabel('Precision [m/s]')
    ax3.set_xlabel("Bands")
    ax3.set_xlim(0.5, len(bands_total) + 0.5)
    ax3.set_xticks(range(1, len(bands_total) + 1))
    ax3.set_xticklabels(bands_total)
    ax3.set_ylim(
        positiony_min - 0.1 * (positiony_max - positiony_min),
        positiony_max + 0.1 * (positiony_max - positiony_min),
    )

    ax3.tick_params(axis="both", which="major", labelsize=15)

    ax4 = fig.add_subplot(224)

    ax4.fill_between(
        range(1, len(bands_total) + 1),
        rv_cumulative(y4_60k_bottom),
        rv_cumulative(y4_60k_top),
        color="b",
        alpha=0.2,
    )
    ax4.fill_between(
        range(1, len(bands_total) + 1),
        rv_cumulative(y4_80k_bottom),
        rv_cumulative(y4_80k_top),
        color="g",
        alpha=0.2,
    )
    ax4.fill_between(
        range(1, len(bands_total) + 1),
        rv_cumulative(y4_100k_bottom),
        rv_cumulative(y4_100k_top),
        color="r",
        alpha=0.2,
    )

    ax4.scatter(
        range(1, len(bands_total) + 1),
        rv_cumulative(y4_60k_bottom),
        marker="^",
        color="b",
        alpha=0.4,
    )
    ax4.scatter(
        range(1, len(bands_total) + 1),
        rv_cumulative(y4_60k_top),
        marker="o",
        color="b",
        alpha=0.4,
    )

    ax4.scatter(
        range(1, len(bands_total) + 1),
        rv_cumulative(y4_80k_bottom),
        marker="^",
        color="g",
        alpha=0.4,
    )
    ax4.scatter(
        range(1, len(bands_total) + 1),
        rv_cumulative(y4_80k_top),
        marker="o",
        color="g",
        alpha=0.4,
    )

    ax4.scatter(
        range(1, len(bands_total) + 1),
        rv_cumulative(y4_100k_bottom),
        marker="^",
        color="r",
        alpha=0.4,
    )
    ax4.scatter(
        range(1, len(bands_total) + 1),
        rv_cumulative(y4_100k_top),
        marker="o",
        color="r",
        alpha=0.4,
    )

    ax4.text(1.0, positiony_max, "M9", size=14)

    # ticks and labels
    ax4.set_xlabel("Bands")
    ax4.set_xlim(0.5, len(bands_total) + 0.5)
    ax4.set_xticks(range(1, len(bands_total) + 1))
    ax4.set_xticklabels(bands_total)
    ax4.set_yticklabels([])
    ax4.set_ylim(
        positiony_min - 0.1 * (positiony_max - positiony_min),
        positiony_max + 0.1 * (positiony_max - positiony_min),
    )

    ax4.tick_params(axis="both", which="major", labelsize=15)

    fig.text(
        0.06,
        0.5,
        r"Precision [m/s]",
        ha="center",
        va="center",
        rotation="vertical",
        size=14,
    )
    fig.subplots_adjust(hspace=0, wspace=0)

    plt.show()
    plt.close()

    """
    VSINI=5km/s
    """
    print("Results for vsini of 5.0 km/s")

    # preparation of data: plot1
    y1_60k = [
        results["M0-Z-5.0-60k"],
        results["M0-Y-5.0-60k"],
        results["M0-J-5.0-60k"],
        results["M0-H-5.0-60k"],
        results["M0-K-5.0-60k"],
    ]
    y1_60k_top = [y[1] for y in y1_60k]
    y1_60k_bottom = [y[2] for y in y1_60k]

    y1_80k = [
        results["M0-Z-5.0-80k"],
        results["M0-Y-5.0-80k"],
        results["M0-J-5.0-80k"],
        results["M0-H-5.0-80k"],
        results["M0-K-5.0-80k"],
    ]
    y1_80k_top = [y[1] for y in y1_80k]
    y1_80k_bottom = [y[2] for y in y1_80k]

    y1_100k = [
        results["M0-Z-5.0-100k"],
        results["M0-Y-5.0-100k"],
        results["M0-J-5.0-100k"],
        results["M0-H-5.0-100k"],
        results["M0-K-5.0-100k"],
    ]
    y1_100k_top = [y[1] for y in y1_100k]
    y1_100k_bottom = [y[2] for y in y1_100k]

    y1_60k_lim = [y[0] for y in y1_60k]

    y1_80k_lim = [y[0] for y in y1_80k]

    y1_100k_lim = [y[0] for y in y1_100k]

    # preparation of data: plot2
    y2_60k = [
        results["M3-Z-5.0-60k"],
        results["M3-Y-5.0-60k"],
        results["M3-J-5.0-60k"],
        results["M3-H-5.0-60k"],
        results["M3-K-5.0-60k"],
    ]
    y2_60k_top = [y[1] for y in y2_60k]
    y2_60k_bottom = [y[2] for y in y2_60k]

    y2_80k = [
        results["M3-Z-5.0-80k"],
        results["M3-Y-5.0-80k"],
        results["M3-J-5.0-80k"],
        results["M3-H-5.0-80k"],
        results["M3-K-5.0-80k"],
    ]
    y2_80k_top = [y[1] for y in y2_80k]
    y2_80k_bottom = [y[2] for y in y2_80k]

    y2_100k = [
        results["M3-Z-5.0-100k"],
        results["M3-Y-5.0-100k"],
        results["M3-J-5.0-100k"],
        results["M3-H-5.0-100k"],
        results["M3-K-5.0-100k"],
    ]
    y2_100k_top = [y[1] for y in y2_100k]
    y2_100k_bottom = [y[2] for y in y2_100k]

    y2_60k_lim = [y[0] for y in y2_60k]

    y2_80k_lim = [y[0] for y in y2_80k]

    y2_100k_lim = [y[0] for y in y2_100k]

    # preparation of data: plot3
    y3_60k = [
        results["M6-Z-5.0-60k"],
        results["M6-Y-5.0-60k"],
        results["M6-J-5.0-60k"],
        results["M6-H-5.0-60k"],
        results["M6-K-5.0-60k"],
    ]
    y3_60k_top = [y[1] for y in y3_60k]
    y3_60k_bottom = [y[2] for y in y3_60k]

    y3_80k = [
        results["M6-Z-5.0-80k"],
        results["M6-Y-5.0-80k"],
        results["M6-J-5.0-80k"],
        results["M6-H-5.0-80k"],
        results["M6-K-5.0-80k"],
    ]
    y3_80k_top = [y[1] for y in y3_80k]
    y3_80k_bottom = [y[2] for y in y3_80k]

    y3_100k = [
        results["M6-Z-5.0-100k"],
        results["M6-Y-5.0-100k"],
        results["M6-J-5.0-100k"],
        results["M6-H-5.0-100k"],
        results["M6-K-5.0-100k"],
    ]
    y3_100k_top = [y[1] for y in y3_100k]
    y3_100k_bottom = [y[2] for y in y3_100k]

    y3_60k_lim = [y[0] for y in y3_60k]

    y3_80k_lim = [y[0] for y in y3_80k]

    y3_100k_lim = [y[0] for y in y3_100k]

    # preparation of data: plot4
    y4_60k = [
        results["M9-Z-5.0-60k"],
        results["M9-Y-5.0-60k"],
        results["M9-J-5.0-60k"],
        results["M9-H-5.0-60k"],
        results["M9-K-5.0-60k"],
    ]
    y4_60k_top = [y[1] for y in y4_60k]
    y4_60k_bottom = [y[2] for y in y4_60k]

    y4_80k = [
        results["M9-Z-5.0-80k"],
        results["M9-Y-5.0-80k"],
        results["M9-J-5.0-80k"],
        results["M9-H-5.0-80k"],
        results["M9-K-5.0-80k"],
    ]
    y4_80k_top = [y[1] for y in y4_80k]
    y4_80k_bottom = [y[2] for y in y4_80k]

    y4_100k = [
        results["M9-Z-5.0-100k"],
        results["M9-Y-5.0-100k"],
        results["M9-J-5.0-100k"],
        results["M9-H-5.0-100k"],
        results["M9-K-5.0-100k"],
    ]
    y4_100k_top = [y[1] for y in y4_100k]
    y4_100k_bottom = [y[2] for y in y4_100k]

    y4_60k_lim = [y[0] for y in y4_60k]

    y4_80k_lim = [y[0] for y in y4_80k]

    y4_100k_lim = [y[0] for y in y4_100k]

    datasets = [
        y1_60k_top,
        y1_60k_bottom,
        y1_80k_top,
        y1_80k_bottom,
        y1_100k_top,
        y1_100k_bottom,
        y2_60k_top,
        y2_60k_bottom,
        y2_80k_top,
        y2_80k_bottom,
        y2_100k_top,
        y2_100k_bottom,
        y3_60k_top,
        y3_60k_bottom,
        y3_80k_top,
        y3_80k_bottom,
        y3_100k_top,
        y3_100k_bottom,
        y4_60k_top,
        y4_60k_bottom,
        y4_80k_top,
        y4_80k_bottom,
        y4_100k_top,
        y4_100k_bottom,
    ]

    positiony_min, poisition_ymax = min_max_y(datasets)

    """
    # data for correction plot
    y1_60k_vsini5 = (np.array(y1_60k_top) - np.array(y1_60k_bottom)) / np.array(y1_60k_bottom)*100.0
    y1_80k_vsini5 = (np.array(y1_80k_top) - np.array(y1_80k_bottom)) / np.array(y1_80k_bottom)*100.0
    y1_100k_vsini5 = (np.array(y1_100k_top) - np.array(y1_100k_bottom)) / np.array(y1_100k_bottom)*100.0

    y2_60k_vsini5 = (np.array(y2_60k_top) - np.array(y2_60k_bottom)) / np.array(y2_60k_bottom)*100.0
    y2_80k_vsini5 = (np.array(y2_80k_top) - np.array(y2_80k_bottom)) / np.array(y2_80k_bottom)*100.0
    y2_100k_vsini5 = (np.array(y2_100k_top) - np.array(y2_100k_bottom)) / np.array(y2_100k_bottom)*100.0

    y3_60k_vsini5 = (np.array(y3_60k_top) - np.array(y3_60k_bottom)) / np.array(y3_60k_bottom)*100.0
    y3_80k_vsini5 = (np.array(y3_80k_top) - np.array(y3_80k_bottom)) / np.array(y3_80k_bottom)*100.0
    y3_100k_vsini5 = (np.array(y3_100k_top) - np.array(y3_100k_bottom)) / np.array(y3_100k_bottom)*100.0

    y4_60k_vsini5 = (np.array(y4_60k_top) - np.array(y4_60k_bottom)) / np.array(y4_60k_bottom)*100.0
    y4_80k_vsini5 = (np.array(y4_80k_top) - np.array(y4_80k_bottom)) / np.array(y4_80k_bottom)*100.0
    y4_100k_vsini5 = (np.array(y4_100k_top) - np.array(y4_100k_bottom)) / np.array(y4_100k_bottom)*100.0
    """

    fig = plt.figure(2)
    ax1 = fig.add_subplot(221)

    ax1.fill_between(
        range(1, len(bands) + 1), y1_60k_bottom, y1_60k_top, color="b", alpha=0.2
    )
    ax1.fill_between(
        range(1, len(bands) + 1), y1_80k_bottom, y1_80k_top, color="g", alpha=0.2
    )
    ax1.fill_between(
        range(1, len(bands) + 1), y1_100k_bottom, y1_100k_top, color="r", alpha=0.2
    )

    ax1.plot(range(1, len(bands) + 1), y1_60k_lim, color="b", linestyle="--")
    ax1.plot(range(1, len(bands) + 1), y1_80k_lim, color="g", linestyle="--")
    ax1.plot(range(1, len(bands) + 1), y1_100k_lim, color="r", linestyle="--")

    ax1.scatter(
        range(1, len(bands) + 1), y1_60k_bottom, marker="^", color="b", alpha=0.4
    )
    ax1.scatter(range(1, len(bands) + 1), y1_60k_top, marker="o", color="b", alpha=0.4)

    ax1.scatter(
        range(1, len(bands) + 1), y1_80k_bottom, marker="^", color="g", alpha=0.4
    )
    ax1.scatter(range(1, len(bands) + 1), y1_80k_top, marker="o", color="g", alpha=0.4)

    ax1.scatter(
        range(1, len(bands) + 1), y1_100k_bottom, marker="^", color="r", alpha=0.4
    )
    ax1.scatter(range(1, len(bands) + 1), y1_100k_top, marker="o", color="r", alpha=0.4)

    ax1.text(1.0, positiony_max, "M0", size=14)

    # ticks and labels
    # ax1.set_ylabel('Precision [m/s]')
    ax1.set_xlim(0.5, len(bands) + 0.5)
    ax1.set_xticks(range(1, len(bands) + 1))
    ax1.set_xticklabels([])
    ax1.set_ylim(
        positiony_min - 0.1 * (positiony_max - positiony_min),
        positiony_max + 0.1 * (positiony_max - positiony_min),
    )

    ax1.tick_params(axis="both", which="major", labelsize=15)

    ax2 = fig.add_subplot(222)

    ax2.fill_between(
        range(1, len(bands) + 1), y2_60k_bottom, y2_60k_top, color="b", alpha=0.2
    )
    ax2.fill_between(
        range(1, len(bands) + 1), y2_80k_bottom, y2_80k_top, color="g", alpha=0.2
    )
    ax2.fill_between(
        range(1, len(bands) + 1), y2_100k_bottom, y2_100k_top, color="r", alpha=0.2
    )

    ax2.plot(range(1, len(bands) + 1), y2_60k_lim, color="b", linestyle="--")
    ax2.plot(range(1, len(bands) + 1), y2_80k_lim, color="g", linestyle="--")
    ax2.plot(range(1, len(bands) + 1), y2_100k_lim, color="r", linestyle="--")

    ax2.scatter(
        range(1, len(bands) + 1), y2_60k_bottom, marker="^", color="b", alpha=0.4
    )
    ax2.scatter(range(1, len(bands) + 1), y2_60k_top, marker="o", color="b", alpha=0.4)

    ax2.scatter(
        range(1, len(bands) + 1), y2_80k_bottom, marker="^", color="g", alpha=0.4
    )
    ax2.scatter(range(1, len(bands) + 1), y2_80k_top, marker="o", color="g", alpha=0.4)

    ax2.scatter(
        range(1, len(bands) + 1), y2_100k_bottom, marker="^", color="r", alpha=0.4
    )
    ax2.scatter(range(1, len(bands) + 1), y2_100k_top, marker="o", color="r", alpha=0.4)

    ax2.text(1.0, positiony_max, "M3", size=14)

    # ticks and labels
    ax2.set_xlim(0.5, len(bands) + 0.5)
    ax2.set_xticks(range(1, len(bands) + 1))
    ax2.set_xticklabels([])
    ax2.set_yticklabels([])
    ax2.set_ylim(
        positiony_min - 0.1 * (positiony_max - positiony_min),
        positiony_max + 0.1 * (positiony_max - positiony_min),
    )

    ax2.tick_params(axis="both", which="major", labelsize=15)

    ax3 = fig.add_subplot(223)

    ax3.fill_between(
        range(1, len(bands) + 1), y3_60k_bottom, y3_60k_top, color="b", alpha=0.2
    )
    ax3.fill_between(
        range(1, len(bands) + 1), y3_80k_bottom, y3_80k_top, color="g", alpha=0.2
    )
    ax3.fill_between(
        range(1, len(bands) + 1), y3_100k_bottom, y3_100k_top, color="r", alpha=0.2
    )

    ax3.plot(range(1, len(bands) + 1), y3_60k_lim, color="b", linestyle="--")
    ax3.plot(range(1, len(bands) + 1), y3_80k_lim, color="g", linestyle="--")
    ax3.plot(range(1, len(bands) + 1), y3_100k_lim, color="r", linestyle="--")

    ax3.scatter(
        range(1, len(bands) + 1), y3_60k_bottom, marker="^", color="b", alpha=0.4
    )
    ax3.scatter(range(1, len(bands) + 1), y3_60k_top, marker="o", color="b", alpha=0.4)

    ax3.scatter(
        range(1, len(bands) + 1), y3_80k_bottom, marker="^", color="g", alpha=0.4
    )
    ax3.scatter(range(1, len(bands) + 1), y3_80k_top, marker="o", color="g", alpha=0.4)

    ax3.scatter(
        range(1, len(bands) + 1), y3_100k_bottom, marker="^", color="r", alpha=0.4
    )
    ax3.scatter(range(1, len(bands) + 1), y3_100k_top, marker="o", color="r", alpha=0.4)

    ax3.text(1.0, positiony_max, "M6", size=14)

    # ticks and labels
    # ax3.set_ylabel('Precision [m/s]')
    ax3.set_xlabel("Bands")
    ax3.set_xlim(0.5, len(bands) + 0.5)
    ax3.set_xticks(range(1, len(bands) + 1))
    ax3.set_xticklabels(bands)
    ax3.set_ylim(
        positiony_min - 0.1 * (positiony_max - positiony_min),
        positiony_max + 0.1 * (positiony_max - positiony_min),
    )

    ax3.tick_params(axis="both", which="major", labelsize=15)

    ax4 = fig.add_subplot(224)

    ax4.fill_between(
        range(1, len(bands) + 1), y4_60k_bottom, y4_60k_top, color="b", alpha=0.2
    )
    ax4.fill_between(
        range(1, len(bands) + 1), y4_80k_bottom, y4_80k_top, color="g", alpha=0.2
    )
    ax4.fill_between(
        range(1, len(bands) + 1), y4_100k_bottom, y4_100k_top, color="r", alpha=0.2
    )

    ax4.plot(range(1, len(bands) + 1), y4_60k_lim, color="b", linestyle="--")
    ax4.plot(range(1, len(bands) + 1), y4_80k_lim, color="g", linestyle="--")
    ax4.plot(range(1, len(bands) + 1), y4_100k_lim, color="r", linestyle="--")

    ax4.scatter(
        range(1, len(bands) + 1), y4_60k_bottom, marker="^", color="b", alpha=0.4
    )
    ax4.scatter(range(1, len(bands) + 1), y4_60k_top, marker="o", color="b", alpha=0.4)

    ax4.scatter(
        range(1, len(bands) + 1), y4_80k_bottom, marker="^", color="g", alpha=0.4
    )
    ax4.scatter(range(1, len(bands) + 1), y4_80k_top, marker="o", color="g", alpha=0.4)

    ax4.scatter(
        range(1, len(bands) + 1), y4_100k_bottom, marker="^", color="r", alpha=0.4
    )
    ax4.scatter(range(1, len(bands) + 1), y4_100k_top, marker="o", color="r", alpha=0.4)

    ax4.text(1.0, positiony_max, "M9", size=14)

    # ticks and labels
    ax4.set_xlabel("Bands")
    ax4.set_xlim(0.5, len(bands) + 0.5)
    ax4.set_xticks(range(1, len(bands) + 1))
    ax4.set_xticklabels(bands)
    ax4.set_yticklabels([])
    ax4.set_ylim(
        positiony_min - 0.1 * (positiony_max - positiony_min),
        positiony_max + 0.1 * (positiony_max - positiony_min),
    )

    ax4.tick_params(axis="both", which="major", labelsize=15)

    fig.text(
        0.06,
        0.5,
        r"Precision [m/s]",
        ha="center",
        va="center",
        rotation="vertical",
        size=14,
    )
    fig.subplots_adjust(hspace=0, wspace=0)

    plt.show()
    plt.close()

    """
    VSINI=10km/s
    """
    print("Results for vsini of 10.0 km/s")

    # preparation of data: plot1
    y1_60k = [
        results["M0-Z-10.0-60k"],
        results["M0-Y-10.0-60k"],
        results["M0-J-10.0-60k"],
        results["M0-H-10.0-60k"],
        results["M0-K-10.0-60k"],
    ]
    y1_60k_top = [y[1] for y in y1_60k]
    y1_60k_bottom = [y[2] for y in y1_60k]

    y1_80k = [
        results["M0-Z-10.0-80k"],
        results["M0-Y-10.0-80k"],
        results["M0-J-10.0-80k"],
        results["M0-H-10.0-80k"],
        results["M0-K-10.0-80k"],
    ]
    y1_80k_top = [y[1] for y in y1_80k]
    y1_80k_bottom = [y[2] for y in y1_80k]

    y1_100k = [
        results["M0-Z-10.0-100k"],
        results["M0-Y-10.0-100k"],
        results["M0-J-10.0-100k"],
        results["M0-H-10.0-100k"],
        results["M0-K-10.0-100k"],
    ]
    y1_100k_top = [y[1] for y in y1_100k]
    y1_100k_bottom = [y[2] for y in y1_100k]

    y1_60k_lim = [y[0] for y in y1_60k]

    y1_80k_lim = [y[0] for y in y1_80k]

    y1_100k_lim = [y[0] for y in y1_100k]

    # preparation of data: plot2
    y2_60k = [
        results["M3-Z-10.0-60k"],
        results["M3-Y-10.0-60k"],
        results["M3-J-10.0-60k"],
        results["M3-H-10.0-60k"],
        results["M3-K-10.0-60k"],
    ]
    y2_60k_top = [y[1] for y in y2_60k]
    y2_60k_bottom = [y[2] for y in y2_60k]

    y2_80k = [
        results["M3-Z-10.0-80k"],
        results["M3-Y-10.0-80k"],
        results["M3-J-10.0-80k"],
        results["M3-H-10.0-80k"],
        results["M3-K-10.0-80k"],
    ]
    y2_80k_top = [y[1] for y in y2_80k]
    y2_80k_bottom = [y[2] for y in y2_80k]

    y2_100k = [
        results["M3-Z-10.0-100k"],
        results["M3-Y-10.0-100k"],
        results["M3-J-10.0-100k"],
        results["M3-H-10.0-100k"],
        results["M3-K-10.0-100k"],
    ]
    y2_100k_top = [y[1] for y in y2_100k]
    y2_100k_bottom = [y[2] for y in y2_100k]

    y2_60k_lim = [y[0] for y in y2_60k]

    y2_80k_lim = [y[0] for y in y2_80k]

    y2_100k_lim = [y[0] for y in y2_100k]

    # preparation of data: plot3
    y3_60k = [
        results["M6-Z-10.0-60k"],
        results["M6-Y-10.0-60k"],
        results["M6-J-10.0-60k"],
        results["M6-H-10.0-60k"],
        results["M6-K-10.0-60k"],
    ]
    y3_60k_top = [y[1] for y in y3_60k]
    y3_60k_bottom = [y[2] for y in y3_60k]

    y3_80k = [
        results["M6-Z-10.0-80k"],
        results["M6-Y-10.0-80k"],
        results["M6-J-10.0-80k"],
        results["M6-H-10.0-80k"],
        results["M6-K-10.0-80k"],
    ]
    y3_80k_top = [y[1] for y in y3_80k]
    y3_80k_bottom = [y[2] for y in y3_80k]

    y3_100k = [
        results["M6-Z-10.0-100k"],
        results["M6-Y-10.0-100k"],
        results["M6-J-10.0-100k"],
        results["M6-H-10.0-100k"],
        results["M6-K-10.0-100k"],
    ]
    y3_100k_top = [y[1] for y in y3_100k]
    y3_100k_bottom = [y[2] for y in y3_100k]

    y3_60k_lim = [y[0] for y in y3_60k]

    y3_80k_lim = [y[0] for y in y3_80k]

    y3_100k_lim = [y[0] for y in y3_100k]

    # preparation of data: plot4
    y4_60k = [
        results["M9-Z-10.0-60k"],
        results["M9-Y-10.0-60k"],
        results["M9-J-10.0-60k"],
        results["M9-H-10.0-60k"],
        results["M9-K-10.0-60k"],
    ]
    y4_60k_top = [y[1] for y in y4_60k]
    y4_60k_bottom = [y[2] for y in y4_60k]

    y4_80k = [
        results["M9-Z-10.0-80k"],
        results["M9-Y-10.0-80k"],
        results["M9-J-10.0-80k"],
        results["M9-H-10.0-80k"],
        results["M9-K-10.0-80k"],
    ]
    y4_80k_top = [y[1] for y in y4_80k]
    y4_80k_bottom = [y[2] for y in y4_80k]

    y4_100k = [
        results["M9-Z-10.0-100k"],
        results["M9-Y-10.0-100k"],
        results["M9-J-10.0-100k"],
        results["M9-H-10.0-100k"],
        results["M9-K-10.0-100k"],
    ]
    y4_100k_top = [y[1] for y in y4_100k]
    y4_100k_bottom = [y[2] for y in y4_100k]

    y4_60k_lim = [y[0] for y in y4_60k]

    y4_80k_lim = [y[0] for y in y4_80k]

    y4_100k_lim = [y[0] for y in y4_100k]

    datasets = [
        y1_60k_top,
        y1_60k_bottom,
        y1_80k_top,
        y1_80k_bottom,
        y1_100k_top,
        y1_100k_bottom,
        y2_60k_top,
        y2_60k_bottom,
        y2_80k_top,
        y2_80k_bottom,
        y2_100k_top,
        y2_100k_bottom,
        y3_60k_top,
        y3_60k_bottom,
        y3_80k_top,
        y3_80k_bottom,
        y3_100k_top,
        y3_100k_bottom,
        y4_60k_top,
        y4_60k_bottom,
        y4_80k_top,
        y4_80k_bottom,
        y4_100k_top,
        y4_100k_bottom,
    ]

    positiony_min, poisition_ymax = min_max_y(datasets)

    # data for correction plot
    y1_60k_vsini10 = (
        (np.array(y1_60k_top) - np.array(y1_60k_bottom))
        / np.array(y1_60k_bottom)
        * 100.0
    )
    y1_80k_vsini10 = (
        (np.array(y1_80k_top) - np.array(y1_80k_bottom))
        / np.array(y1_80k_bottom)
        * 100.0
    )
    y1_100k_vsini10 = (
        (np.array(y1_100k_top) - np.array(y1_100k_bottom))
        / np.array(y1_100k_bottom)
        * 100.0
    )

    y2_60k_vsini10 = (
        (np.array(y2_60k_top) - np.array(y2_60k_bottom))
        / np.array(y2_60k_bottom)
        * 100.0
    )
    y2_80k_vsini10 = (
        (np.array(y2_80k_top) - np.array(y2_80k_bottom))
        / np.array(y2_80k_bottom)
        * 100.0
    )
    y2_100k_vsini10 = (
        (np.array(y2_100k_top) - np.array(y2_100k_bottom))
        / np.array(y2_100k_bottom)
        * 100.0
    )

    y3_60k_vsini10 = (
        (np.array(y3_60k_top) - np.array(y3_60k_bottom))
        / np.array(y3_60k_bottom)
        * 100.0
    )
    y3_80k_vsini10 = (
        (np.array(y3_80k_top) - np.array(y3_80k_bottom))
        / np.array(y3_80k_bottom)
        * 100.0
    )
    y3_100k_vsini10 = (
        (np.array(y3_100k_top) - np.array(y3_100k_bottom))
        / np.array(y3_100k_bottom)
        * 100.0
    )

    y4_60k_vsini10 = (
        (np.array(y4_60k_top) - np.array(y4_60k_bottom))
        / np.array(y4_60k_bottom)
        * 100.0
    )
    y4_80k_vsini10 = (
        (np.array(y4_80k_top) - np.array(y4_80k_bottom))
        / np.array(y4_80k_bottom)
        * 100.0
    )
    y4_100k_vsini10 = (
        (np.array(y4_100k_top) - np.array(y4_100k_bottom))
        / np.array(y4_100k_bottom)
        * 100.0
    )

    fig = plt.figure(3)
    ax1 = fig.add_subplot(221)

    ax1.fill_between(
        range(1, len(bands) + 1), y1_60k_bottom, y1_60k_top, color="b", alpha=0.2
    )
    ax1.fill_between(
        range(1, len(bands) + 1), y1_80k_bottom, y1_80k_top, color="g", alpha=0.2
    )
    ax1.fill_between(
        range(1, len(bands) + 1), y1_100k_bottom, y1_100k_top, color="r", alpha=0.2
    )

    ax1.plot(range(1, len(bands) + 1), y1_60k_lim, color="b", linestyle="--")
    ax1.plot(range(1, len(bands) + 1), y1_80k_lim, color="g", linestyle="--")
    ax1.plot(range(1, len(bands) + 1), y1_100k_lim, color="r", linestyle="--")

    ax1.scatter(
        range(1, len(bands) + 1), y1_60k_bottom, marker="^", color="b", alpha=0.4
    )
    ax1.scatter(range(1, len(bands) + 1), y1_60k_top, marker="o", color="b", alpha=0.4)

    ax1.scatter(
        range(1, len(bands) + 1), y1_80k_bottom, marker="^", color="g", alpha=0.4
    )
    ax1.scatter(range(1, len(bands) + 1), y1_80k_top, marker="o", color="g", alpha=0.4)

    ax1.scatter(
        range(1, len(bands) + 1), y1_100k_bottom, marker="^", color="r", alpha=0.4
    )
    ax1.scatter(range(1, len(bands) + 1), y1_100k_top, marker="o", color="r", alpha=0.4)

    ax1.text(1.0, positiony_max, "M0", size=14)

    # ticks and labels
    # ax1.set_ylabel('Precision [m/s]')
    ax1.set_xlim(0.5, len(bands) + 0.5)
    ax1.set_xticks(range(1, len(bands) + 1))
    ax1.set_xticklabels([])
    ax1.set_ylim(
        positiony_min - 0.1 * (positiony_max - positiony_min),
        positiony_max + 0.1 * (positiony_max - positiony_min),
    )

    ax1.tick_params(axis="both", which="major", labelsize=15)

    ax2 = fig.add_subplot(222)

    ax2.fill_between(
        range(1, len(bands) + 1), y2_60k_bottom, y2_60k_top, color="b", alpha=0.2
    )
    ax2.fill_between(
        range(1, len(bands) + 1), y2_80k_bottom, y2_80k_top, color="g", alpha=0.2
    )
    ax2.fill_between(
        range(1, len(bands) + 1), y2_100k_bottom, y2_100k_top, color="r", alpha=0.2
    )

    ax2.plot(range(1, len(bands) + 1), y2_60k_lim, color="b", linestyle="--")
    ax2.plot(range(1, len(bands) + 1), y2_80k_lim, color="g", linestyle="--")
    ax2.plot(range(1, len(bands) + 1), y2_100k_lim, color="r", linestyle="--")

    ax2.scatter(
        range(1, len(bands) + 1), y2_60k_bottom, marker="^", color="b", alpha=0.4
    )
    ax2.scatter(range(1, len(bands) + 1), y2_60k_top, marker="o", color="b", alpha=0.4)

    ax2.scatter(
        range(1, len(bands) + 1), y2_80k_bottom, marker="^", color="g", alpha=0.4
    )
    ax2.scatter(range(1, len(bands) + 1), y2_80k_top, marker="o", color="g", alpha=0.4)

    ax2.scatter(
        range(1, len(bands) + 1), y2_100k_bottom, marker="^", color="r", alpha=0.4
    )
    ax2.scatter(range(1, len(bands) + 1), y2_100k_top, marker="o", color="r", alpha=0.4)

    ax2.text(1.0, positiony_max, "M3", size=14)

    # ticks and labels
    ax2.set_xlim(0.5, len(bands) + 0.5)
    ax2.set_xticks(range(1, len(bands) + 1))
    ax2.set_xticklabels([])
    ax2.set_yticklabels([])
    ax2.set_ylim(
        positiony_min - 0.1 * (positiony_max - positiony_min),
        positiony_max + 0.1 * (positiony_max - positiony_min),
    )

    ax2.tick_params(axis="both", which="major", labelsize=15)

    ax3 = fig.add_subplot(223)

    ax3.fill_between(
        range(1, len(bands) + 1), y3_60k_bottom, y3_60k_top, color="b", alpha=0.2
    )
    ax3.fill_between(
        range(1, len(bands) + 1), y3_80k_bottom, y3_80k_top, color="g", alpha=0.2
    )
    ax3.fill_between(
        range(1, len(bands) + 1), y3_100k_bottom, y3_100k_top, color="r", alpha=0.2
    )

    ax3.plot(range(1, len(bands) + 1), y3_60k_lim, color="b", linestyle="--")
    ax3.plot(range(1, len(bands) + 1), y3_80k_lim, color="g", linestyle="--")
    ax3.plot(range(1, len(bands) + 1), y3_100k_lim, color="r", linestyle="--")

    ax3.scatter(
        range(1, len(bands) + 1), y3_60k_bottom, marker="^", color="b", alpha=0.4
    )
    ax3.scatter(range(1, len(bands) + 1), y3_60k_top, marker="o", color="b", alpha=0.4)

    ax3.scatter(
        range(1, len(bands) + 1), y3_80k_bottom, marker="^", color="g", alpha=0.4
    )
    ax3.scatter(range(1, len(bands) + 1), y3_80k_top, marker="o", color="g", alpha=0.4)

    ax3.scatter(
        range(1, len(bands) + 1), y3_100k_bottom, marker="^", color="r", alpha=0.4
    )
    ax3.scatter(range(1, len(bands) + 1), y3_100k_top, marker="o", color="r", alpha=0.4)

    ax3.text(1.0, positiony_max, "M6", size=14)

    # ticks and labels
    # ax3.set_ylabel('Precision [m/s]')
    ax3.set_xlabel("Bands")
    ax3.set_xlim(0.5, len(bands) + 0.5)
    ax3.set_xticks(range(1, len(bands) + 1))
    ax3.set_xticklabels(bands)
    ax3.set_ylim(
        positiony_min - 0.1 * (positiony_max - positiony_min),
        positiony_max + 0.1 * (positiony_max - positiony_min),
    )

    ax3.tick_params(axis="both", which="major", labelsize=15)

    ax4 = fig.add_subplot(224)

    ax4.fill_between(
        range(1, len(bands) + 1), y4_60k_bottom, y4_60k_top, color="b", alpha=0.2
    )
    ax4.fill_between(
        range(1, len(bands) + 1), y4_80k_bottom, y4_80k_top, color="g", alpha=0.2
    )
    ax4.fill_between(
        range(1, len(bands) + 1), y4_100k_bottom, y4_100k_top, color="r", alpha=0.2
    )

    ax4.plot(range(1, len(bands) + 1), y4_60k_lim, color="b", linestyle="--")
    ax4.plot(range(1, len(bands) + 1), y4_80k_lim, color="g", linestyle="--")
    ax4.plot(range(1, len(bands) + 1), y4_100k_lim, color="r", linestyle="--")

    ax4.scatter(
        range(1, len(bands) + 1), y4_60k_bottom, marker="^", color="b", alpha=0.4
    )
    ax4.scatter(range(1, len(bands) + 1), y4_60k_top, marker="o", color="b", alpha=0.4)

    ax4.scatter(
        range(1, len(bands) + 1), y4_80k_bottom, marker="^", color="g", alpha=0.4
    )
    ax4.scatter(range(1, len(bands) + 1), y4_80k_top, marker="o", color="g", alpha=0.4)

    ax4.scatter(
        range(1, len(bands) + 1), y4_100k_bottom, marker="^", color="r", alpha=0.4
    )
    ax4.scatter(range(1, len(bands) + 1), y4_100k_top, marker="o", color="r", alpha=0.4)

    ax4.text(1.0, positiony_max, "M9", size=14)

    # ticks and labels
    ax4.set_xlabel("Bands")
    ax4.set_xlim(0.5, len(bands) + 0.5)
    ax4.set_xticks(range(1, len(bands) + 1))
    ax4.set_xticklabels(bands)
    ax4.set_yticklabels([])
    ax4.set_ylim(
        positiony_min - 0.1 * (positiony_max - positiony_min),
        positiony_max + 0.1 * (positiony_max - positiony_min),
    )

    ax4.tick_params(axis="both", which="major", labelsize=15)

    fig.text(
        0.06,
        0.5,
        r"Precision [m/s]",
        ha="center",
        va="center",
        rotation="vertical",
        size=14,
    )
    fig.subplots_adjust(hspace=0, wspace=0)

    plt.show()
    plt.close()

    # correction figure

    datasets = [
        y1_60k_vsini1,
        y1_60k_vsini10,
        y1_80k,
        y1_80k_vsini10,
        y1_100k_vsini1,
        y1_100k_vsini10,
        y2_60k_vsini1,
        y2_60k_vsini10,
        y2_80k_vsini1,
        y2_80k_vsini10,
        y2_100k_vsini1,
        y2_100k_vsini10,
        y3_60k_vsini1,
        y3_60k_vsini10,
        y3_80k_vsini1,
        y3_80k_vsini10,
        y3_100k_vsini1,
        y3_100k_vsini10,
        y4_60k_vsini1,
        y4_60k_vsini10,
        y4_80k_vsini1,
        y4_80k_vsini10,
        y4_100k_vsini1,
        y4_100k_vsini10,
    ]

    positiony_min, poisition_ymax = min_max_y(datasets)
    rc("xtick", labelsize=15)
    rc("ytick", labelsize=15)

    fig = plt.figure(4)
    ax1 = fig.add_subplot(221)

    ax1.scatter(
        range(1, len(bands) + 1),
        y1_60k_vsini1,
        marker="<",
        s=100.0,
        color="b",
        alpha=0.2,
    )
    ax1.scatter(
        range(1, len(bands) + 1),
        y1_80k_vsini1,
        marker="<",
        s=100.0,
        color="g",
        alpha=0.2,
    )
    ax1.scatter(
        range(1, len(bands) + 1),
        y1_100k_vsini1,
        marker="<",
        s=100.0,
        color="r",
        alpha=0.2,
    )

    ax1.scatter(
        range(1, len(bands) + 1),
        y1_60k_vsini10,
        marker=">",
        s=100.0,
        color="b",
        alpha=0.2,
    )
    ax1.scatter(
        range(1, len(bands) + 1),
        y1_80k_vsini10,
        marker=">",
        s=100.0,
        color="g",
        alpha=0.2,
    )
    ax1.scatter(
        range(1, len(bands) + 1),
        y1_100k_vsini10,
        marker=">",
        s=100.0,
        color="r",
        alpha=0.2,
    )

    ax1.text(1.0, positiony_max * 0.90, "M0", size=14)

    # ticks and labels
    ax1.set_xlim(0.5, len(bands) + 0.5)
    ax1.set_xticks(range(1, len(bands) + 1))
    ax1.set_xticklabels([])
    ax1.set_ylim(
        positiony_min - 0.05 * (positiony_max - positiony_min),
        positiony_max + 0.05 * (positiony_max - positiony_min),
    )

    ax2 = fig.add_subplot(222)

    ax2.scatter(
        range(1, len(bands) + 1),
        y2_60k_vsini1,
        marker="<",
        s=100.0,
        color="b",
        alpha=0.2,
    )
    ax2.scatter(
        range(1, len(bands) + 1),
        y2_80k_vsini1,
        marker="<",
        s=100.0,
        color="g",
        alpha=0.2,
    )
    ax2.scatter(
        range(1, len(bands) + 1),
        y2_100k_vsini1,
        marker="<",
        s=100.0,
        color="r",
        alpha=0.2,
    )

    ax2.scatter(
        range(1, len(bands) + 1),
        y2_60k_vsini10,
        marker=">",
        s=100.0,
        color="b",
        alpha=0.2,
    )
    ax2.scatter(
        range(1, len(bands) + 1),
        y2_80k_vsini10,
        marker=">",
        s=100.0,
        color="g",
        alpha=0.2,
    )
    ax2.scatter(
        range(1, len(bands) + 1),
        y2_100k_vsini10,
        marker=">",
        s=100.0,
        color="r",
        alpha=0.2,
    )

    ax2.text(1.0, positiony_max * 0.90, "M3", size=14)

    # ticks and labels
    ax2.set_xlim(0.5, len(bands) + 0.5)
    ax2.set_xticks(range(1, len(bands) + 1))
    ax2.set_xticklabels([])
    ax2.set_yticklabels([])
    ax2.set_ylim(
        positiony_min - 0.05 * (positiony_max - positiony_min),
        positiony_max + 0.05 * (positiony_max - positiony_min),
    )

    ax3 = fig.add_subplot(223)

    ax3.scatter(
        range(1, len(bands) + 1),
        y3_60k_vsini1,
        marker="<",
        s=100.0,
        color="b",
        alpha=0.2,
    )
    ax3.scatter(
        range(1, len(bands) + 1),
        y3_80k_vsini1,
        marker="<",
        s=100.0,
        color="g",
        alpha=0.2,
    )
    ax3.scatter(
        range(1, len(bands) + 1),
        y3_100k_vsini1,
        marker="<",
        s=100.0,
        color="r",
        alpha=0.2,
    )

    ax3.scatter(
        range(1, len(bands) + 1),
        y3_60k_vsini10,
        marker=">",
        s=100.0,
        color="b",
        alpha=0.2,
    )
    ax3.scatter(
        range(1, len(bands) + 1),
        y3_80k_vsini10,
        marker=">",
        s=100.0,
        color="g",
        alpha=0.2,
    )
    ax3.scatter(
        range(1, len(bands) + 1),
        y3_100k_vsini10,
        marker=">",
        s=100.0,
        color="r",
        alpha=0.2,
    )

    ax3.text(1.0, positiony_max * 0.90, "M6", size=14)

    # ticks and labels
    ax3.set_xlabel("Bands")
    ax3.set_xlim(0.5, len(bands) + 0.5)
    ax3.set_xticks(range(1, len(bands) + 1))
    ax3.set_xticklabels(bands)
    ax3.set_ylim(
        positiony_min - 0.05 * (positiony_max - positiony_min),
        positiony_max + 0.05 * (positiony_max - positiony_min),
    )

    ax4 = fig.add_subplot(224)

    ax4.scatter(
        range(1, len(bands) + 1),
        y4_60k_vsini1,
        marker="<",
        s=100.0,
        color="b",
        alpha=0.2,
    )
    ax4.scatter(
        range(1, len(bands) + 1),
        y4_80k_vsini1,
        marker="<",
        s=100.0,
        color="g",
        alpha=0.2,
    )
    ax4.scatter(
        range(1, len(bands) + 1),
        y4_100k_vsini1,
        marker="<",
        s=100.0,
        color="r",
        alpha=0.2,
    )

    ax4.scatter(
        range(1, len(bands) + 1),
        y4_60k_vsini10,
        marker=">",
        s=100.0,
        color="b",
        alpha=0.2,
    )
    ax4.scatter(
        range(1, len(bands) + 1),
        y4_80k_vsini10,
        marker=">",
        s=100.0,
        color="g",
        alpha=0.2,
    )
    ax4.scatter(
        range(1, len(bands) + 1),
        y4_100k_vsini10,
        marker=">",
        s=100.0,
        color="r",
        alpha=0.2,
    )

    ax4.text(1.0, positiony_max * 0.90, "M9", size=14)

    # ticks and labels
    ax4.set_xlabel("Bands")
    ax4.set_xlim(0.5, len(bands) + 0.5)
    ax4.set_xticks(range(1, len(bands) + 1))
    ax4.set_xticklabels(bands)
    ax4.set_yticklabels([])
    ax4.set_ylim(
        positiony_min - 0.05 * (positiony_max - positiony_min),
        positiony_max + 0.05 * (positiony_max - positiony_min),
    )

    fig.text(
        0.06,
        0.5,
        r"Loss in RV precision [\%]",
        ha="center",
        va="center",
        rotation="vertical",
        size=14,
    )
    fig.subplots_adjust(hspace=0, wspace=0)

    plt.show()
    plt.close()

    # if the paper_plots option is on, then print the results
    print("\n")
    for star in spectral_types:
        for band in bands:
            for vel in vsini:
                for res in R:
                    for smpl in sampling:
                        id_string = "{0:s}-{1:s}-{2:.01f}-{3:s}-{4:2.01f}".format(
                            star, band, float(vel), res, float(smpl)
                        )
                        precision = results[id_string]
                        print(
                            "{0!s}: & {1:.1f}\t & {2:.1f}\t & {3:.1f} \\\\".format(
                                id_string, precision[0], precision[1], precision[2]
                            )
                        )


def min_max_y(datasets):
    """Get min and max y values for joint plotting."""
    y_max = np.max([np.max(data) for data in datasets])
    y_min = np.min([np.min(data) for data in datasets])
    return y_min, y_max



#!/usr/bin/env python
"""Prepare_data.py

Process the Phoenix-aces synthetic spectra and create .dat files with
wavelength and flux.
Adds them to the data directory of eniric for convolutions etc.
This wastes a lot of memory duplicating wavelength vector.

Jason Neal January 2017
"""
import argparse
import os
import re
import sys

import numpy as np
from astropy.io import fits

import eniric
import eniric.io_module as io


def _parser():
    """Take care of all the argparse stuff.

    :returns: the args
    """
    parser = argparse.ArgumentParser(
        description="Transform spectra to prepare for convolution."
    )
    parser.add_argument(
        "-s",
        "--startype",
        help='Spectral Type e.g "MO"',
        default=["M0"],
        type=str,
        nargs="+",
    )
    parser.add_argument(
        "-t",
        "--temp",
        help="Temperature of stars to prepare",
        type=float,
        nargs="+",
        default=[3900.0],
        choices=list(np.arange(2300, 7000, 100.0))
        + list(np.arange(7000, 12001, 200.0)),
    )
    parser.add_argument(
        "-l",
        "--logg",
        help="Logg for stellar models.",
        default=[4.50],
        type=float,
        nargs="+",
        choices=np.arange(0, 6.01, 0.5),
    )
    parser.add_argument(
        "-m",
        "--metallicity",
        type=float,
        default=[0.0],
        help="Metallicity values.",
        nargs="+",
    )
    # choices=[list(np.arange(-4.0, -2.0, 1))+list(np.arange(-2.0, 1.01, 0.5))]
    parser.add_argument(
        "-a",
        "--alpha",
        type=float,
        default=[0.0],
        choices=np.arange(-0.2, 1.201, 0.2),
        help="Alpha values. Default = [0.0]",
        nargs="+",
    )
    parser.add_argument(
        "-f",
        "--flux_type",
        type=str,
        default="photon",
        choices=["photon", "energy"],
        help="Type of flux to use. Default converts it to photons.",
    )
    parser.add_argument(
        "-d",
        "--data_dir",
        help="Data directory to save results.",
        type=str,
        default=None,
    )
    parser.add_argument(
        "-p",
        "--phoenix_dir",
        default=None,
        type=str,
        help="Phoenix directory to find fits files",
    )
    parser.add_argument(
        "-r",
        "--replace",
        action="store_true",
        help="Replace data files if already created.",
    )

    return parser.parse_args()


def main(
    startype,
    temp,
    logg,
    metallicity,
    alpha,
    flux_type="photon",
    data_dir=None,
    phoenix_dir=None,
    replace=False,
):
    """Prepare datafiles for phoenix models that match the input parameters.

    This add the wavelength information to each spectra and converts
    to microns/photons if the flux_type="photons" is given.
    We do realise that this is a waste of space and it would be more
    storage efficient to just read in the phoenix raw fits files and
    wavelength file when needed.

    """
    if data_dir is None:
        data_dir = eniric.paths["phoenix_dat"]
    os.makedirs(data_dir, exist_ok=True)

    if phoenix_dir is None:
        phoenix_dir = eniric.paths["phoenix_raw"]

    # Get Phoenix wavelength data
    wavelength_file = "WAVE_PHOENIX-ACES-AGSS-COND-2011.fits"
    wavelength = fits.getdata(os.path.join(phoenix_dir, wavelength_file))

    if flux_type == "photon":
        file_suffix = "_wave_photon.dat"  # For saving output
    else:
        file_suffix = "_wave.dat"

    # Convert Stellar_types into
    stellar_dict = {"M0": 3900.0, "M3": 3500.0, "M6": 2800.0, "M9": 2600.0}
    # Add temperature of stellar_type to temp list
    for star in startype:
        try:
            temp.append(stellar_dict[star])
        except KeyError:
            print(
                "Stellar type {0} is not implemented here (yet), submit and issue.".format(
                    star
                )
            )

    # Get all phoenix fits files we want to convert
    for (path, dirs, files) in os.walk(phoenix_dir):

        phoenix_files = []
        for f in files:
            # Test if filename meets conditions
            end_cond = f.endswith("PHOENIX-ACES-AGSS-COND-2011-HiRes.fits")

            try:
                if "Alpha=" in f:
                    (match_temp, match_logg, match_feh, match_alpha) = re.search(
                        r"(\d{5})-(\d\.\d\d)([+\-]\d\.\d)\.Alpha=([+\-]\d\.\d\d)\.", f
                    ).groups()
                    alpha_cond = float(match_alpha) in alpha
                else:
                    (match_temp, match_logg, match_feh) = re.search(
                        r"(\d{5})-(\d\.\d\d)([+\-]\d\.\d)", f
                    ).groups()
                    alpha_cond = True  # To make work
            except AttributeError:
                # This file doesn't match what we want so continue with next loop
                continue

            temp_cond = float(match_temp) in temp
            feh_cond = float(match_feh) in metallicity
            logg_cond = float(match_logg) in logg

            if np.all(
                [end_cond, temp_cond, feh_cond, logg_cond, alpha_cond]
            ):  # All conditions met
                # Matching file found
                phoenix_files.append(f)
            else:
                pass

        for phoenix_file in phoenix_files:
            z_folder = path.split(os.sep)[-1]
            os.makedirs(os.path.join(data_dir, z_folder), exist_ok=True)
            output_filename = os.path.join(
                data_dir, z_folder, phoenix_file[:-5] + file_suffix
            )  # Name of .dat file
            if os.path.exists(output_filename) and not replace:
                print(
                    "Skipping as {0} already exists (use -r to replace)".format(
                        output_filename
                    )
                )
                continue
            spectra = fits.getdata(os.path.join(path, phoenix_file))

            # Need to add conversions pedro preformed to flux!
            """The energy units of Phoenix fits files is erg/s/cm**2/cm
            We transform the flux into photons in the read_spectrum()
            function by multiplying the flux result by the wavelength (lambda)

                Flux_photon = Flux_energy/Energy_photon
            with
                Energy_photon = h*c/lambda
            Flux_photon = Flux_energy * lambda / (h * c)

            Here we convert the flux into erg/s/cm**2/\mum by multiplying by 10**-4 cm/\mum
            Flux_e(erg/s/cm**2/\mum)  = Flux_e(erg/s/cm**2/cm) * (1 cm) / (10000 \mum)
            """

            spectra_micron = spectra * 10 ** -4  # Convert   /cm    to  /micron

            if flux_type == "photon":
                wavelength_micron = (
                    wavelength * 10 ** -4
                )  # Convert Angstrom to   micron

                spectra_photon = (
                    spectra_micron * wavelength_micron
                )  # Ignoring constants h*c in photon energy equation

                result = io.pdwrite_cols(
                    output_filename,
                    wavelength_micron,
                    spectra_photon,
                    header=["# Wavelength (micron)", r"Flux (photon/s/cm^2)"],
                    float_format="%.7f",
                )

            else:
                result = io.pdwrite_cols(
                    output_filename,
                    wavelength,
                    spectra_micron,
                    header=["# Wavelength (Angstrom)", r"Flux (erg/s/cm^2/micron)"],
                    float_format=None,
                )

            if not result:
                print("Successfully wrote to ", output_filename)
            else:
                print("Failed to write to ", output_filename)

    print("Done")
    return 0


if __name__ == "__main__":
    args = vars(_parser())
    opts = {k: args[k] for k in args}
    sys.exit(main(**opts))



"""Calculate the Radial Velocity Precision of NIR Spectra.

    Notes
    -----
    Extract from https://arxiv.org/pdf/1511.07468v1.pdf
    From Eq. (11) and (12) of
    https://arxiv.org/pdf/1511.07468v1.pdf#cite.2001A%26A...374..733B
    it follows that the RV uncertainty associated with the information content
    of a given spectra is given by

        RV_rms = c /(Q *Sum{Ne}) = c / (sqrt{sum{W(i)}})

    in which c is the speed of light in vacuum, Q the quality factor of a
    spectrum, and N_e the total number of photoelectrons collected inside the
    wavelength range of interest. However, the precision can only be calculated
    using the concept of optimal pixel weight W(i) for each of the pixels i
    that compose the spectra.

        W(i) = lambda(i)**2 (d'A_0(i) / d'lambda(i))**2 / (A_0(i) + sigma_D**2)

    in which lambda(i) and A_0(i) are the values of each pixel wave-length and
    flux, respectively. The weight will be proportional to the information
    content of the spectrum, given by the derivative of the amplitude, and
    calculated following Connes (1985).
    The denominator of the previous equation is the variance of the flux of
    the pixel A_0(i), depending on the flux value itself and on the
    detector noise sigma_D. In this paper we exclusively consider the high
    signal-to-noise ratio regime, so we can approximate A_0(i) + sigma_D**2
    to A_0(i).

    Spectral Quality:
        Q = sqrt{sum{W(i)}} / sqrt{sum{A_0{i}}

    The spectral quality, Q, is independent of the flux level and is only
    a function of the spectral profile.

    Notes
    -----
    Extract from https://arxiv.org/pdf/1511.07468v1.pdf

    The precision can only be calculated using the concept of
    optimal pixel weight W(i) for each of the pixels i that compose the spectra.

        W(i) = lambda(i)**2 (d'A_0(i) / d'lambda(i))**2 / (A_0(i) + sigma_D**2)

    in which lambda(i) and A_0(i) are the values of each pixel wave-length and
    flux, respectively. The weight will be proportional to the information
    content of the spectrum, given by the derivative of the amplitude, and
    calculated following Connes (1985).

"""
import warnings
from typing import Optional, Tuple, Union

import astropy.constants as const
import astropy.units as u
import numpy as np
from astropy.units.quantity import Quantity
from numpy import ndarray

from eniric.resample import log_chunks

c = const.c


def rv_precision(
    wavelength: Union[Quantity, ndarray],
    flux: Union[Quantity, ndarray],
    mask: Optional[ndarray] = None,
    **kwargs,
) -> Quantity:
    """Calculate the theoretical RV precision achievable on a spectrum.

    Parameters
    ----------
    wavelength: array-like or Quantity array
        Wavelength of spectrum.
    flux: array-like or Quantity array
        Flux of spectrum.
    mask:Optional[ndarray]
        Masking function array to apply to the pixel weights.
    kwargs:
        Kwargs for sqrt_sum_wis

    Returns
    -------
    RVrms : Quantity scalar
       Radial velocity precision of spectra in m/s.

    """
    return c / sqrt_sum_wis(wavelength, flux, mask=mask, **kwargs)


def quality(
    wavelength: Union[Quantity, ndarray], flux: Union[Quantity, ndarray], **kwargs
) -> Union[float, Quantity]:
    """Calculation of the spectral Quality, Q, for a spectrum.

    Parameters
    ----------
    wavelength: array-like or Quantity array
        Wavelength of spectrum.
    flux: array-like or Quantity array
        Flux of spectrum.
    kwargs:
        Kwargs for sqrt_sum_wis

    Returns
    -------
    sqrt{sum{W(i)}}: float
       Spectral quality

    Notes
    -----
        Q = sqrt{sum{W(i)}} / sqrt{sum{A_0{i}}

    The spectral quality, Q, is independent of the flux level and is only
    a function of the spectral profile.

    """
    flux = flux * u.dimensionless_unscaled  # Turn into Quantity if not already
    flux = flux / flux.unit  # Remove units from flux (sqrt(N_e) is unitless)

    wis = sqrt_sum_wis(wavelength, flux, **kwargs)
    q = wis / np.sqrt(np.nansum(flux))
    return q.value


def sqrt_sum_wis(
    wavelength: Union[Quantity, ndarray],
    flux: Union[Quantity, ndarray],
    mask: Optional[Union[Quantity, ndarray]] = None,
    grad: bool = True,
) -> Union[float, Quantity]:
    """Calculation of the Square root of the sum of the weights(Wis) for a spectrum.

    Mask is used to apply a masking function to the weights (to mask out telluric lines for example)

        W(i) = W(i) * m(i)

    Parameters
    ----------
    wavelength: array-like or Quantity array
        Wavelength of spectrum.
    flux: array-like or Quantity array
        Flux of spectrum.
    mask: Optional ndarray
        Weighting mask function. Default is all ones.
    grad: bool
        Flag to use np.gradient. Default=True. Original publication used less precise method.

    Returns
    -------
    sqrt{sum{W(i)}}: float or Quantity scalar
       Square root of the sum of the pixel weights(Wis)

    """
    if mask is None:
        # Don't use np.ones_like() as it will take units of a Quantity.
        mask = np.ones(len(wavelength))

    mask_check(mask)

    pixel_wis = pixel_weights(wavelength, flux, grad=grad)

    # Apply masking function
    if grad:
        masked_wis = pixel_wis * mask
    else:
        masked_wis = pixel_wis * mask[:-1]

    sqrt_sum = np.sqrt(np.nansum(masked_wis))
    if not np.isfinite(sqrt_sum):
        warnings.warn("Weight sum is not finite = {}".format(sqrt_sum))
    if sqrt_sum == 0:
        warnings.warn(
            "Sum of weights sum is = {}. This will cause infinite errors.".format(
                sqrt_sum
            )
        )
    return sqrt_sum


def mask_check(mask):
    """Checks for mask array."""
    if isinstance(mask, u.Quantity):
        if not (mask.unit == u.dimensionless_unscaled):
            raise TypeError(
                "Mask should not be a non-dimensionless and unscaled Quantity!"
            )
        mask = mask.value

    # Check for values of mask
    if np.any(mask > 1) or np.any(mask < 0):
        raise ValueError("Mask should be within range from 0 to 1 only.")


def slope(wavelength, flux):
    """Forward Finite difference derivative which looses one value of array.

        f' = (f(x+h)-f(x)) / h.
        f'[i] = (flux[i+1] - flux[i])/ (wavelength[i+1] - wavelength[i])

    Returns
    -------
    Array with n-1 points.
    """
    return np.diff(flux) / np.diff(wavelength)


def pixel_weights(
    wavelength: Union[ndarray, Quantity],
    flux: Union[ndarray, Quantity],
    grad: bool = True,
):
    """Calculate individual pixel weights.
    w(i) = \lambda(i)^2 (\partial A(i)/\partial\lambda)^2 / A(i)

    Parameters
    ----------
    wavelength: Union[ndarray, Quantity]
        Wavelength array.
    flux: Union[ndarray, Quantity]
     Flux array.
    grad: bool
        Toggle function for spectral slope. Default False + forward finite difference.
    """
    if isinstance(flux, u.Quantity):
        """Units of variance are squared"""
        flux_variance = flux.value * flux.unit * flux.unit
    else:
        flux_variance = flux

    dydx_unit = 1
    if grad:
        # Hack for quantities with numpy gradient
        if isinstance(flux, Quantity):
            dydx_unit *= flux.unit
            flux = flux.value
        if isinstance(wavelength, Quantity):
            dydx_unit /= wavelength.unit
            wave = wavelength.value
        else:
            wave = wavelength
        derivf_over_lambda = np.gradient(flux, wave) * dydx_unit

        return (wavelength * derivf_over_lambda) ** 2.0 / flux_variance
    else:
        derivf_over_lambda = slope(wavelength, flux)
        return (wavelength[:-1] * derivf_over_lambda) ** 2.0 / flux_variance[:-1]


def incremental_quality(
    wavelength: ndarray, flux: ndarray, percent: Union[int, float] = 10, **kwargs
) -> Tuple[ndarray, ndarray]:
    """Determine spectral quality in incremental sections.

    Parameters
    ----------
    wavelength: array-like or Quantity array
        Wavelength of spectrum.
    flux: array-like or Quantity array
        Flux of spectrum.
    percent: Union[int,float]  (default=10)
        The percent size of chunk around each wavelength position.
    kwargs:
        Kwargs passed onto quality().

    Returns
    -------
    x: ndarray
     Central wavelength value for quality section.
    q: ndarray
       Spectral quality for each section.
    """
    positions = log_chunks(wavelength, percent)
    qualities = []
    for pos1, pos2 in zip(positions[:-1], positions[1:]):
        mask = (wavelength >= pos1) & (wavelength < pos2)
        x = wavelength[mask]
        y = flux[mask]
        q = quality(x, y, **kwargs)
        qualities.append([np.mean(x), q])
    x, q = np.asarray(qualities).T
    return x, q


def incremental_rv(
    wavelength: ndarray, flux: ndarray, mask: ndarray, percent: float = 10, **kwargs
) -> Tuple[ndarray, ndarray]:
    """Determine spectral RV precision in incremental sections.
    Parameters
    ----------
    wavelength: array-like or Quantity array
        Wavelength of spectrum.
    flux: array-like or Quantity array
        Flux of spectrum.
    flux: array-like or Quantity array
        Pixel weight mask.
    percent: Union[int,float]  (default=10)
        The percent size of chunk around each wavelength position.
    kwargs:
        Kwargs passed onto quality().

    Returns
    -------
    x: ndarray
     Central wavelength value for quality section.
    rv: ndarray
       Spectral RV precision for each section.
    """
    positions = log_chunks(wavelength, percent)
    velocities = []
    for pos1, pos2 in zip(positions[:-1], positions[1:]):
        pos_mask = (wavelength >= pos1) & (wavelength < pos2)
        x = wavelength[pos_mask]
        y = flux[pos_mask]
        if mask is not None:
            z = mask[pos_mask]
        else:
            z = mask  # None
        rv_calc = rv_precision(x, y, z, **kwargs)
        velocities.append([np.mean(x), rv_calc.value])

    x, rv = np.asarray(velocities).T
    return x, rv



import warnings
from os.path import join
from typing import List, Optional

import numpy as np
from astropy import constants as const
from numpy import ndarray

import eniric
import eniric.io_module as io
from eniric.broaden import resolution_convolution
from eniric.utilities import band_limits


class Atmosphere(object):
    """Atmospheric transmission object.

    Stores wavelength and atmospheric transmission arrays.
    Enables telluric masking and accounting for barycentric motion.

    Attributes
    ----------
    wl: ndarray
        Wavelength array
    transmission: ndarray
        Atmospheric transmission (between 0 and 1)
    std: ndarray
        Standard deviation of transmission.
    mask: ndarray
        Transmission mask (1's are kept)
    shifted: bool
        Indicate shifted mask

    Constructors
    ----------
    from_file(atmmodel)
        Read in atmospheric model and prepare.
    from_band(band, bary=False)
        Read in atmospheric model for given band.

    Methods
    -------
    to_file(fname, header, fmt)
        Save the atmospheric model to a txt file.
    at(wave)
        Return the transmission value at the closest points to wave.
    wave_select(wl_min, wl_max)
       Slice Atmosphere between two wavelengths.
    band_select(band)
        Slice Atmosphere to a given band.
    copy()
        Make a copy of atmosphere object.
    mask_transmission(depth)
        Mask the transmission below given depth. e.g. 2%
    bary_shift_mask(rv, consecutive_test)
        Sweep telluric mask symmetrically by rv.
    broaden(resolution, *kwargs)
        Instrument broadening of the atmospheric transmission profile.

    Configuration
    -------------
    Two things can be set for the Atmosphere class in the `config.yaml` file
    The path to atmosphere data
    e.g.
        paths:
            atmmodel: "path/to/atmmodel/directory"
    The name for the atmosphere model .txt file
        atmmodel:
            base: "Average_TAPAS_2014"
    """

    def __init__(self, wavelength, transmission, mask=None, std=None, shifted=False):
        assert len(wavelength) == len(
            transmission
        ), "Wavelength and transmission do not match length."
        self.wl = np.asarray(wavelength)
        self.transmission = np.asarray(transmission)
        if std is None:
            self.std = np.zeros_like(wavelength)
        else:
            self.std = np.asarray(std)
        if mask is None:
            self.mask = np.ones_like(wavelength, dtype=bool)
        else:
            self.mask = np.asarray(mask, dtype=bool)
        self.shifted = shifted

    @classmethod
    def from_file(cls, atmmodel: str):
        """Read in atmospheric model and prepare.

        Alternate constructor for Atmosphere.

        Parameters
        ----------
        atmmodel: str
            Name of atmosphere file.
        """
        wav_atm, flux_atm, std_flux_atm, mask_atm = io.pdread_4col(atmmodel)
        wav_atm = wav_atm / 1e3  # conversion from nanometers to micrometers
        mask_atm = np.array(mask_atm, dtype=bool)
        # We do not use the std from the year atmosphere but need it for compatibility.
        shifted = True if "_bary" in atmmodel else False
        return cls(
            wavelength=wav_atm,
            transmission=flux_atm,
            mask=mask_atm,
            std=std_flux_atm,
            shifted=shifted,
        )

    @classmethod
    def from_band(cls, band: str, bary: bool = False):
        """Read in atmospheric model for given band.

        Alternate constructor for Atmosphere. Base on "base" path in config.yaml.

        Parameters
        ----------
        band: str
            Name of atmosphere file.
        bary: bool
            Barycentric shifted mask.
        """

        extension = "_bary.txt" if bary else ".txt"
        atmmodel = join(
            eniric.paths["atmmodel"],
            "{0}_{1}{2}".format(eniric.atmmodel["base"], band, extension),
        )

        try:
            # Try find the band file
            atm = cls.from_file(atmmodel)
        except IOError:
            warnings.warn(
                """Could not find band file for band {0}.
             It is recommend to create this using
                `split_atmosphere.py -b {0}`
                `bary_shift_atmmodel.py -b {0}`
             Trying to load main atmosphere file for now. (will be slower).""".format(
                    band
                )
            )
            full_model = join(
                eniric.paths["atmmodel"], "{0}.txt".format(eniric.atmmodel["base"])
            )
            atm = cls.from_file(full_model)

            # Shorten to band
            atm = atm.wave_select(*band_limits(band))
            if bary:
                atm.bary_shift_mask(consecutive_test=True)
        return atm

    def to_file(
        self, fname: str, header: Optional[List[str]] = None, fmt: str = "%11.8f"
    ):
        """Save the atmospheric model to a txt file.

        Converts micron back into nanometers to be consistent with from_file().

        Parameters
        ----------
        fname: str
            Name of atmosphere file to save to.
        header:
            Header lines to add.
        fmt: str
             String formatting
        """
        if header is None:
            header = ["# atm_wav(nm)", "atm_flux", "atm_std_flux", "atm_mask"]
        return io.pdwrite_cols(
            fname,
            self.wl * 1000,
            self.transmission,
            self.std,
            self.mask.astype(int),
            header=header,
            float_format=fmt,
        )

    def __getitem__(self, item):
        """Index Atmosphere by returning a Atmosphere with indexed components."""
        return Atmosphere(
            wavelength=self.wl[item],
            transmission=self.transmission[item],
            mask=self.mask[item],
            std=self.std[item],
        )

    def at(self, wave):
        """Return the transmission value at the closest points to wave.

        This assumes that the atmosphere model is
        sampled much higher than the stellar spectra.

        For instance the default has a sampling if 10 compared to 3.
        (instead of interpolation)

        Parameters
        ----------
        wave: ndarray
            Wavelengths at which to return closest atmosphere values.
        """
        # Getting the wav, flux and mask values from the atm model
        # that are the closest to the stellar wav values, see
        # https://stackoverflow.com/questions/2566412/find-nearest-value-in-numpy-array
        index_atm = np.searchsorted(self.wl, wave)
        wl_len = len(self.wl)
        # replace indexes outside the array, at the very end, by the value at the very end
        # index_atm = [index if(index < len(wav_atm)) else len(wav_atm)-1 for index in index_atm]
        index_mask = index_atm >= wl_len  # find broken indices
        index_atm[index_mask] = wl_len - 1  # replace with index of end.
        return self[index_atm]

    def wave_select(self, wl_min, wl_max):
        """Slice Atmosphere between two wavelengths."""
        wl_mask = (self.wl < wl_max) & (self.wl > wl_min)
        return self[wl_mask]

    def band_select(self, band):
        """Slice Atmosphere to a given Band."""
        wl_min, wl_max = band_limits(band)
        return self.wave_select(wl_min, wl_max)

    def copy(self):
        """Make a copy of atmosphere object."""
        return Atmosphere(
            wavelength=self.wl.copy(),
            transmission=self.transmission.copy(),
            mask=self.mask.copy(),
            std=self.std.copy(),
        )

    def mask_transmission(self, depth: float = 2.0) -> None:
        """Mask the transmission below given depth. e.g. 2%

        Parameters
        ----------
        depth : float (default = 2.0)
            Telluric line depth percentage to mask out.
            E.g. depth=2 will mask transmission deeper than 2%.

        Updates the mask.
        """
        cutoff = 1 - depth / 100.0
        self.mask = self.transmission >= cutoff

    def bary_shift_mask(self, rv: float = 30.0, consecutive_test: bool = False):
        """Sweep telluric mask symmetrically by rv.

        Parameters
        ----------
        rv: float (default=30 km/s)
            Barycentric RV to extend masks in km/s. (Default=30 km/s)
        consecutive_test: bool (default False)
            Checks for 3 consecutive zeros to mask out transmission.

        """
        if self.shifted:
            warnings.warn(
                "Detected that 'shifted' is already True. "
                "Check that you want to rv extend masks again."
            )
        rv_mps = rv * 1e3  # Convert from km/s into m/s

        shift_amplitudes = self.wl * rv_mps / const.c.value
        # Operate element wise
        blue_shifts = self.wl - shift_amplitudes
        red_shifts = self.wl + shift_amplitudes

        bary_mask = []
        for (blue_wl, red_wl, mask) in zip(blue_shifts, red_shifts, self.mask):
            if mask == 0:
                this_mask_value = False
            else:
                # np.searchsorted is faster then the boolean masking wavelength range
                # It returns index locations to put the min/max doppler-shifted values
                slice_limits = np.searchsorted(self.wl, [blue_wl, red_wl])
                slice_limits = [
                    index if (index < len(self.wl)) else len(self.wl) - 1
                    for index in slice_limits
                ]  # Fix searchsorted end index

                mask_slice = self.mask[slice_limits[0] : slice_limits[1]]

                if consecutive_test:
                    # Mask value False if 3 or more consecutive zeros in slice.
                    len_consec_zeros = consecutive_truths(~mask_slice)
                    if np.all(
                        ~mask_slice
                    ):  # All pixels of slice is zeros (shouldn't get here)
                        this_mask_value = False
                    elif np.max(len_consec_zeros) >= 3:
                        this_mask_value = False
                    else:
                        this_mask_value = True
                        if np.sum(~mask_slice) > 3:
                            print(
                                (
                                    "There were {0}/{1} zeros in this "
                                    "barycentric shift but None were 3 consecutive!"
                                ).format(np.sum(~mask_slice), len(mask_slice))
                            )

                else:
                    this_mask_value = np.bool(
                        np.product(mask_slice)
                    )  # Any 0s will make it 0

                # Checks
                if not this_mask_value:
                    assert np.any(~mask_slice)
                else:
                    if not consecutive_test:
                        assert np.all(mask_slice)
            bary_mask.append(this_mask_value)
        self.mask = np.asarray(bary_mask, dtype=np.bool)
        self.shifted = True

    def broaden(self, resolution: float, fwhm_lim: float = 5, num_procs=None):
        """Instrument broadening of the atmospheric transmission profile.

        This does not change any created masks.

        Parameters
        ----------
        resolution: float
            Instrumental resolution/resolving power
        fwhm_lim: int/float
            Number of FWHM to extend convolution.
        num_procs: Optional[int]
            Number of processors to compute the convolution with. Default = total processors - 1
        """
        self.transmission = resolution_convolution(
            wavelength=self.wl,
            extended_wav=self.wl,
            extended_flux=self.transmission,
            R=resolution,
            fwhm_lim=fwhm_lim,
            num_procs=num_procs,
            normalize=True,
        )


def consecutive_truths(condition: ndarray) -> ndarray:
    """Length of consecutive true values in an bool array.

    Parameters
    ----------
    condition: ndarray of bool
        True False array of a condition.

    Returns
    -------
    len_consecutive: ndarray of ints
        Array of lengths of consecutive true values of the condition.

    Notes
    -----
    Solution found at {http://stackoverflow.com/questions/24342047/
                       count-consecutive-occurences-of-values-varying-in-length-in-a-numpy-array}
    """
    if not np.any(condition):  # No match to condition
        return np.array([0])
    else:
        unequal_consec = np.concatenate(
            ([condition[0]], condition[:-1] != condition[1:], [True])
        )
        where_changes = np.where(unequal_consec)[0]  # indices where condition changes
        len_consecutive = np.diff(where_changes)[
            ::2
        ]  # step through every second to get the "True" lengths.
    return len_consecutive



import argparse
import itertools
import os
from os.path import join
from typing import List, Tuple

import multiprocess as mprocess
import numpy as np
from astropy import units as u
from numpy import ndarray

import eniric
from eniric.atmosphere import Atmosphere
from eniric.broaden import convolution
from eniric.corrections import correct_artigau_2018
from eniric.Qcalculator import quality, rv_precision
from eniric.resample import log_resample
from eniric.snr_normalization import snr_constant_band
from eniric.utilities import (
    band_middle,
    doppler_shift_flux,
    load_aces_spectrum,
    load_btsettl_spectrum,
)

num_procs_minus_1 = mprocess.cpu_count() - 1

ref_choices = ["SELF"]
ref_choices.extend(eniric.bands["all"])


def _parser():
    """Take care of all the argparse stuff.

    :returns: the args
    """
    parser = argparse.ArgumentParser(
        description="Calculate quality for any library spectra."
    )
    parser.add_argument("-t", "--temp", type=int, help="Temperature", nargs="+")
    parser.add_argument(
        "-l",
        "--logg",
        type=float,
        default=[4.5],
        help="Logg, default = [4.5]",
        nargs="+",
    )
    parser.add_argument(
        "-m",
        "--metal",
        type=float,
        default=[0.0],
        help="Metallicity, default=[0.0]",
        nargs="+",
    )
    parser.add_argument(
        "-a",
        "--alpha",
        type=float,
        default=[0.0],
        help="Alpha, default=[0.0]",
        nargs="+",
    )
    parser.add_argument(
        "-s", "--sampling", type=float, default=[3.0], help="Sampling", nargs="+"
    )
    parser.add_argument(
        "-r",
        "--resolution",
        type=float,
        default=[50000],
        help="Instrumental resolution",
        nargs="+",
    )
    parser.add_argument(
        "-v",
        "--vsini",
        type=float,
        default=[1.0],
        help="Rotational Velocity",
        nargs="+",
    )
    parser.add_argument(
        "-b",
        "--bands",
        type=str,
        default="J",
        choices=eniric.bands["all"],
        help="Wavelength bands to select. Default=J.",
        nargs="+",
    )
    parser.add_argument(
        "--model",
        type=str,
        default="phoenix",
        choices=["phoenix", "btsettl"],
        help="Spectral models to use. Default=phoenix.",
    )
    parser.add_argument(
        "--save", default=False, action="store_true", help="Save results to file."
    )
    parser.add_argument(
        "--snr", help="Mid-band SNR scaling. (Default=100)", default=100, type=float
    )
    parser.add_argument(
        "--ref_band",
        help="SNR reference band. Default=J. (Default=100). "
        "'self' scales each band relative to the SNR itself.",
        choices=ref_choices,
        default="J",
        type=str,
    )
    parser.add_argument(
        "--num_procs",
        help="Number of processors to use. Default = number of cpus -1",
        default=num_procs_minus_1,
        type=int,
    )
    parser.add_argument(
        "-o",
        "--output",
        help="Filename for results",
        default="quality_results.csv",
        type=str,
    )
    parser.add_argument(
        "--rv", help="Radial velocity value.", default=[0.0], type=float, nargs="+"
    )
    parser.add_argument(
        "--add_rv", help="Include radial velocity shift.", action="store_true"
    )
    parser.add_argument(
        "--air", help="Convert wavelengths from vacuum to air", action="store_true"
    )
    parser.add_argument("--correct", help="Apply RV corrections", action="store_true")
    return parser.parse_args()


def do_analysis(
    star_params,
    vsini: float,
    R: float,
    band: str,
    sampling: float = 3.0,
    conv_kwargs=None,
    snr: float = 100.0,
    ref_band: str = "J",
    rv: float = 0.0,
    air: bool = False,
    model="phoenix",
):
    """Precision and Quality for specific parameter set.

    Parameters
    ----------
    air: bool
        Get model in air wavelengths.
    model: str
        Name of synthetic library to use. (phoenix, btsettl).
    rv: float
        Radial velocity.

    Notes:
        We apply the radial velocity doppler shift after
           - convolution (rotation and resolution)
           - resampling
           - SNR normalization.
        in this way the RV only effects the precision due to the telluric mask interaction.
        The RV should maybe come between the rotational and instrumental convolution
        but we assume this effect is negligible.
    """
    if conv_kwargs is None:
        conv_kwargs = {
            "epsilon": 0.6,
            "fwhm_lim": 5.0,
            "num_procs": 1,
            "normalize": True,
        }

    if ref_band.upper() == "SELF":
        ref_band = band

    if model == "phoenix":
        # Full photon count spectrum
        wav, flux = load_aces_spectrum(star_params, photons=True, air=air)
    elif model == "btsettl":
        wav, flux = load_btsettl_spectrum(star_params, photons=True, air=air)
    else:
        raise ValueError(
            "Model name error in '{}'. Valid choices are 'phoenix and 'btsettl'".format(
                model
            )
        )

    wav_grid, sampled_flux = convolve_and_resample(
        wav, flux, vsini, R, band, sampling, **conv_kwargs
    )

    # Doppler shift
    try:
        if rv != 0:
            sampled_flux = doppler_shift_flux(wav_grid, sampled_flux, vel=rv)
    except Exception as e:
        print("Doppler shift was unsuccessful")
        raise e

    # Spectral Quality
    q = quality(wav_grid, sampled_flux)

    # Scale normalization for precision
    wav_ref, sampled_ref = convolve_and_resample(
        wav, flux, vsini, R, ref_band, sampling, **conv_kwargs
    )
    snr_normalize = snr_constant_band(
        wav_ref, sampled_ref, snr=snr, band=ref_band, sampling=sampling
    )
    sampled_flux = sampled_flux / snr_normalize

    if ref_band == band:
        mid_point = band_middle(ref_band)
        index_ref = np.searchsorted(
            wav_grid, mid_point
        )  # searching for the index closer to 1.25 micron
        snr_estimate = np.sqrt(np.sum(sampled_flux[index_ref - 1 : index_ref + 2]))
        print(
            "\tSanity Check: The S/N at {0:4.02} micron = {1:4.2f}, (should be {2:g}).".format(
                mid_point, snr_estimate, snr
            )
        )

    # Load Atmosphere for this band.
    atm = Atmosphere.from_band(band=band, bary=True).at(wav_grid)
    assert np.allclose(atm.wl, wav_grid), "The atmosphere does not cover the wav_grid"

    # Spectral Quality/Precision
    q = quality(wav_grid, sampled_flux)

    # Precision given by the first condition:
    prec1 = rv_precision(wav_grid, sampled_flux, mask=None)

    # Precision as given by the second condition
    prec2 = rv_precision(wav_grid, sampled_flux, mask=atm.mask)

    # Precision as given by the third condition: M = T**2
    prec3 = rv_precision(wav_grid, sampled_flux, mask=atm.transmission ** 2)

    # Turn quality back into a Quantity (to give it a .value method)
    q = q * u.dimensionless_unscaled
    return [q, prec1, prec2, prec3]


def convolve_and_resample(
    wav: ndarray,
    flux: ndarray,
    vsini: float,
    R: float,
    band: str,
    sampling: float,
    **conv_kwargs,
) -> Tuple[ndarray, ndarray]:
    """Convolve and resample functions together.

    Returns
    -------
    wav_grid: ndarray
        Resampled wavelength array
    sampled_flux: ndarray
        Convolved and resampled flux array
    """
    wav_band, flux_band, convolved_flux = convolution(
        wav, flux, vsini, R, band, **conv_kwargs
    )
    # Re-sample to sampling per resolution element.
    wav_grid = log_resample(wav_band, sampling, R)
    sampled_flux = np.interp(wav_grid, wav_band, convolved_flux)
    return wav_grid, sampled_flux


def model_format_args(model, pars):
    """Format the model and parameter args to save in output file.
    Change to int, str and float.

    model in [temp, logg, fe/h, alpha]
    pars in order (R, band, vsini, sample).

    Can now also optionally handle a 5th parameter rv.
    """
    temp = int(model[0])
    logg = float(model[1])
    fe_h = float(model[2])
    alpha = float(model[3])
    band = pars[1]
    res = int(pars[0] / 1000)
    vsini = float(pars[2])
    sample = float(pars[3])
    try:
        rv = float(pars[4])
        return temp, logg, fe_h, alpha, band, res, vsini, sample, rv
    except IndexError:
        return temp, logg, fe_h, alpha, band, res, vsini, sample


def header_row(add_rv=False):
    """Header row for output file."""
    if add_rv:
        header = (
            "Temp, logg, [Fe/H], Alpha, Band, Resolution, vsini, Sampling, "
            "RV, Quality, Cond. 1, Cond. 2, Cond. 3, correct flag\n"
        )
    else:
        header = (
            "Temp, logg, [Fe/H], Alpha, Band, Resolution, vsini, Sampling, "
            "Quality, Cond. 1, Cond. 2, Cond. 3, correct flag\n"
        )
    return header


def get_already_computed(filename: str, add_rv: bool = False):
    """Get the string of already computer model/parameters from the result file."""
    computed_values = []
    with open(filename, "r") as f:
        for line in f:
            if add_rv:
                # should be longer
                computed_values.append(line[:47])
                computed_values.append(line[:40])
            else:
                computed_values.append(line[:41])
                computed_values.append(line[:34])
    return computed_values


def is_already_computed(computed_values: List[str], model, pars, add_rv: bool = False):
    """Check if any combinations have already been preformed"""
    model_par_str_args = model_format_args(model, pars)

    if add_rv:
        if len(model_par_str_args) != 9:
            raise ValueError("model_par_str_args is incorrect length")

        params_one = (
            "{0:5d}, {1:3.01f}, {2:4.01f}, {3:3.01f}, {4:s}, {5:3d}k,"
            " {6:4.01f}, {7:3.01f}"
        ).format(*model_par_str_args)
        # may change output to have less spaces in future
        params_two = (
            "{0:5d},{1:3.01f},{2:4.01f},{3:3.01f},{4:s},{5:3d}k," "{6:4.01f},{7:3.01f}"
        ).format(*model_par_str_args)

    else:

        model_par_str_args = model_par_str_args[:8]

        if len(model_par_str_args) != 8:
            raise ValueError("model_par_str_args is incorrect length")

        params_one = (
            "{0:5d}, {1:3.01f}, {2:4.01f}, {3:3.01f}, {4:s}, {5:3d}k,"
            " {6:4.01f}, {7:3.01f}"
        ).format(*model_par_str_args)
        # may change output to have less spaces in future
        params_two = (
            "{0:5d},{1:3.01f},{2:4.01f},{3:3.01f},{4:s},{5:3d}k," "{6:4.01f},{7:3.01f}"
        ).format(*model_par_str_args)

    result = (params_one in computed_values) or (params_two in computed_values)
    if result:
        print(model_par_str_args, "model already computed")
        print(params_one)

    return result


if __name__ == "__main__":
    args = _parser()

    # check bt-settl parameters
    if args.model == "btsettl":
        if (args.metal != [0]) or (args.alpha != [0]):
            raise ValueError(
                "You cannot vary metallicity and alpha for BT-Settl, remove these flags."
            )

    try:
        num_procs = args.num_procs
    except AttributeError:
        num_procs = num_procs_minus_1

    try:
        normalize = args.normalzie
    except AttributeError:
        normalize = True

    snr = args.snr
    air = args.air
    if "ALL" in args.bands:
        args.bands.extend(eniric.bands["all"])
        args.bands = set(args.bands)  # Unique
    ref_band = args.ref_band

    conv_kwargs = {
        "epsilon": 0.6,
        "fwhm_lim": 5.0,
        "num_procs": num_procs,
        "normalize": normalize,
    }

    # Load the relevant spectra
    if args.model == "phoenix":
        models_list = itertools.product(args.temp, args.logg, args.metal, args.alpha)
    else:
        models_list = itertools.product(args.temp, args.logg, [0], [0])

    if (args.rv != [0.0]) and (not args.add_rv):
        raise ValueError("Need to include --add_rv flag if a non-zero RV is used.")

    if not os.path.exists(args.output):
        with open(args.output, "a") as f:
            header = header_row(add_rv=args.add_rv)
            f.write(header)

    # Find all model/parameter combinations already computed.
    # To later skip recalculation.
    computed_values = get_already_computed(args.output, args.add_rv)

    with open(args.output, "a") as f:

        for model in models_list:
            # Create generator for params_list
            params_list = itertools.product(
                args.resolution, args.bands, args.vsini, args.sampling, args.rv
            )

            for (R, band, vsini, sample, rv) in params_list:
                pars = (R, band, vsini, sample, rv)

                if is_already_computed(
                    computed_values, model, pars, add_rv=args.add_rv
                ):
                    # skipping the recalculation
                    continue
                else:
                    result = do_analysis(
                        model,
                        vsini=vsini,
                        R=R,
                        band=band,
                        conv_kwargs=conv_kwargs,
                        snr=snr,
                        ref_band=ref_band,
                        sampling=sample,
                        air=air,
                        model=args.model,
                    )
                    result = [
                        round(res.value, 1) if res is not None else None
                        for res in result
                    ]

                    if args.correct:
                        # Apply Artigau 2018 Corrections
                        corr_value = correct_artigau_2018(band)
                        for ii, res in enumerate(result):
                            if ii > 0:  # Not the quality
                                result[ii] = res * corr_value

                    result[0] = int(result[0]) if result[0] is not None else None

                    if args.add_rv:
                        output_template = (
                            "{0:5d}, {1:3.01f}, {2:4.01f}, {3:3.01f}, {4:s}, {5:3d}k,"
                            " {6:4.01f}, {7:3.01f}, {8:3.01f}, {9:6d}, {10:5.01f}, "
                            "{11:5.01f}, {12:5.01f}, {13:1d}\n"
                        )
                        output_model_args = model_format_args(model, pars)
                    else:
                        output_template = (
                            "{0:5d}, {1:3.01f}, {2:4.01f}, {3:3.01f}, {4:s}, {5:3d}k,"
                            " {6:4.01f}, {7:3.01f}, {8:6d}, {9:5.01f}, {10:5.01f}, "
                            "{11:5.01f}, {12:1d}\n"
                        )
                        output_model_args = model_format_args(model, pars)[:8]
                    f.write(
                        output_template.format(
                            *output_model_args,
                            result[0],
                            result[1],
                            result[2],
                            result[3],
                            int(args.correct),
                        )
                    )
    print("Done")



"""Automated snr normalization.

The photon flux can be scaled by multiplicative constant which
affects the SNR of the spectra and the radial velocity precision.

For consistency and valid comparision we normalize each spectra
to achieve a consistent SNR at a specific location.
"""

from typing import Optional, Union

import numpy as np
from numpy import ndarray

import eniric.utilities as utils


def snr_constant_band(
    wav: ndarray,
    flux: ndarray,
    snr: Union[int, float] = 100,
    band: str = "J",
    sampling: Union[int, float] = 3.0,
) -> float:
    """Determine the normalization constant to achieve a SNR in the middle of a given band.

    SNR estimated by the square root of the number of photons in a resolution element.

    Parameters
    ----------
    wav: ndarray
        Wavelength array (microns)
    flux: ndarray
        Photon flux array (photons/s/cm**2)
    snr: int,  default = 100
        SNR to normalize to.
    band: str, default = "J"
        Band to use for normalization.
    sampling: int or float
       Number of pixels per resolution element.

    Returns
    -------
    normalization_value: float
        Normalization value to divide spectrum by to achieve a
        signal-to-noise level of snr within an resolution element
        in the middle of the band.

    Note
    ----
    If sampling is a float it will rounded to the nearest integer for indexing.
    """
    band_middle = utils.band_middle(band)

    if not (wav[0] < band_middle < wav[-1]):
        raise ValueError("Band center not in wavelength range.")

    norm_constant = snr_constant_wav(
        wav, flux, wav_ref=band_middle, snr=snr, sampling=sampling
    )

    return norm_constant


def snr_constant_wav(
    wav: ndarray,
    flux: ndarray,
    wav_ref: float,
    snr: Union[int, float] = 100,
    sampling: Union[int, float] = 3.0,
) -> float:
    """Determine the normalization constant to achieve a SNR at given wavelength.

    SNR estimated by the square root of the number of photons in a resolution element.

    Parameters
    ----------
    wav: ndarray
        Wavelength in micron
    flux: ndarray
        Photon flux array (photons/s/cm**2)
    wav_ref: float
        Wavelength to set the SNR per resolution element.
    snr: int,  default = 100
        SNR to set.
    sampling: int or float
       Number of pixels per resolution element.

    Returns
    -------
    norm_value: float
        Normalization value to divide the flux by to achieve the desired SNR "SNR"
        in resolution element (defined by "sampling") around the wavelength "wav_ref".

    Notes
    -----
    We want to be consistent for each spectra. If we specify the middle of J band
    as the reference it will need to be used for all bands of that spectra.

    If sampling is a float it will rounded to the nearest integer for indexing.
    """
    if wav_ref < wav[0] or wav_ref > wav[-1]:
        raise ValueError("Reference wavelength is outside of the wavelength bounds")
    index_ref = np.searchsorted(wav, [wav_ref])[0]  # Searching for the closest index

    indexes = sampling_index(
        index_ref, sampling=np.round(sampling), array_length=len(wav)
    )

    snr_estimate = np.sqrt(np.sum(flux[indexes]))

    print(
        "\tSanity Check: The reference S/N at {1:3.02f} was of {0:4.2f}.".format(
            snr_estimate, wav_ref
        )
    )
    norm_value = (snr_estimate / snr) ** 2
    return norm_value


def sampling_index(
    index: int, sampling: int = 3, array_length: Optional[int] = None
) -> ndarray:
    """Get a small number of index values around the given index value.

    Parameters
    ----------
    index: int
        The index value which to select values around.
    sampling: int
        Number of index values to return (sampling per resolution element). Must be an integer.
    array_length: int or None, default = None
        Length of array the indexes will be used in. To not exceed array length.

    Returns
    -------
    indexes: ndarray of int64
        The index values.

    """
    import math

    half_sampling = math.floor(sampling / 2)
    if sampling % 2 == 0:  # even sampling
        # index values must be integer
        indexes = np.arange(index - half_sampling, index + half_sampling, dtype=int)
        assert len(indexes) % 2 == 0  # confirm even
        assert len(indexes) == sampling
    else:
        indexes = index + np.arange(-half_sampling, sampling - half_sampling, dtype=int)
        assert len(indexes) % 2 != 0  # confirm odd
        assert len(indexes) == sampling

    if array_length is not None:
        if np.any(indexes >= array_length):
            # This may need fixed up in the future.
            raise ValueError("Indexes has values greater than the length of array.")

    if np.any(indexes < 0):
        raise ValueError("Indexes has values less than 0.")

    return indexes



import os
import re
from typing import Tuple, Union

from numpy.core.multiarray import ndarray

import eniric
from eniric.io_module import pdread_2col
from eniric.snr_normalization import snr_constant_band

resampled_dir = eniric.paths["resampled"]


def normalize_flux(
    flux: ndarray,
    id_string: str,
    new: bool = True,
    snr: Union[int, float] = 100.0,
    ref_band: str = "J",
    sampling: Union[int, float] = 3.0,
) -> ndarray:
    """Normalize flux to have SNR of 100 in middle of reference band.

    Parameters
    ----------
    flux: ndarray
        Photon flux.
    id_string: str
        Identifying string for spectra.
    new: bool default=True
        Choose between new and old constant for testing.
    snr: int, float default=100
        SNR to normalize to, .
    ref_band: str default="J"
        References band to normalize to.
    sampling: int or float
       Number of pixels per resolution element.

    Returns
    -------
    normalized_flux: ndarray
        Flux normalized to a S/N of SNR in the middle of the ref_band.

    """
    # print("Starting norm of {}".format(id_string))
    if new:
        if ref_band.upper() == "SELF":
            __, ref_band, __, __ = decompose_id_string(id_string)

        wav_ref, flux_ref = get_reference_spectrum(id_string, ref_band)
        norm_const = snr_constant_band(
            wav_ref, flux_ref, snr=snr, band=ref_band, sampling=sampling
        )
    else:
        if (ref_band.upper() != "J") or (snr != 100):
            raise ValueError("The old way does not work with these reference values.")
        norm_const = old_norm_constant(id_string) * 1e4  # Input flux offset

    print("{0:s} normalization constant = {1:f}".format(id_string, norm_const))

    return flux / norm_const


def old_norm_constant(id_string: str) -> float:
    """Normalization constants for Figueira et al 2016.

    These are the manual values to achieve a SNR of 100 at 1.25 micro
    for the set of parameter combinations presented.
    """
    if "M0" in id_string and (
        "1.0" in id_string or "5.0" in id_string or "10.0" in id_string
    ):
        norm_constant = 1607
    elif "M3" in id_string and (
        "1.0" in id_string or "5.0" in id_string or "10.0" in id_string
    ):
        norm_constant = 1373
    elif "M6" in id_string and "1.0" in id_string:
        norm_constant = 933
    elif "M6" in id_string and "5.0" in id_string:
        norm_constant = 967
    elif "M6" in id_string and "10.0" in id_string:
        norm_constant = 989
    elif "M9" in id_string and "1.0" in id_string:
        norm_constant = 810
    elif "M9" in id_string and "5.0" in id_string:
        norm_constant = 853
    elif "M9" in id_string and "10.0" in id_string:
        norm_constant = 879
    else:
        print("Constant not defined. Aborting...")
        raise ValueError("Bad ID string")

    return (norm_constant / 100.0) ** 2.0


def decompose_id_string(id_string: str) -> Tuple[str, str, str, str]:
    """Get the values back out of the id-string."""
    match = re.search(r"(M\d)-(\w{1,4})-(\d{1,2}\.0)-(\d{2,3}k)", id_string)
    if match:
        star, band, vel, res = match.groups()
    else:
        raise ValueError(
            "Id-string {0} is not valid for normalization.".format(id_string)
        )

    if band not in eniric.bands["all"]:
        raise ValueError(
            "The band '{}' found in id-string is not valid.".format(id_string)
        )

    return star, band, vel, res


def get_reference_spectrum(
    id_string: str, ref_band: str = "J"
) -> Tuple[ndarray, ndarray]:
    """From the id_string find the correct Spectrum to calculate norm_constant from."""
    # TODO: add option for Alpha into ID-String
    # TODO: Add metallicity and logg into id string
    # TODO: Add metallicity folder

    # Determine the correct reference file to use.
    if ("Alpha=" in id_string) or ("smpl" in id_string):
        raise NotImplementedError
    else:
        star, band, vel, res = decompose_id_string(id_string)

    smpl = 3.0  # Fixed value atm

    ref_band = ref_band.upper()
    if ref_band == "SELF":
        ref_band = band

    file_to_read = (
        "Spectrum_{0}-PHOENIX-ACES_{1}band_vsini{2}_R{3}" "_res{4:3.01f}.txt"
    ).format(star, ref_band, vel, res, float(smpl))

    try:
        wav_ref, flux_ref = pdread_2col(
            os.path.join(eniric.paths["resampled"], file_to_read)
        )
    except FileNotFoundError as e:
        print(
            "The reference spectra in {0:s} band was not found for id {1:s}".format(
                ref_band, id_string
            )
        )
        raise e
    return wav_ref, flux_ref
